/*
 * HistoryReader.java
 *
 * Created on 29 janvier 2004, 23:23
 */

package simulations.core.history;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.FileNotFoundException;

/** This class describes the reader of the history of a game
 * @author tom
 */
public class HistoryReader {
    
    /** The history file **/
    private File file;
    
    /** The file reader of the history file **/
    private FileReader fileReader;
    
    private int offset;
    
    /** Creates a new instance of HistoryReader
     * @param file the history file
     * @throws FileNotFoundException if the file doesn't exist
     */
    public HistoryReader(File file) throws FileNotFoundException {
        this.file = file;
        offset=0;
        //System.out.println(filePath);
        try {
            fileReader = new FileReader(file);
        }catch(FileNotFoundException e) {
            throw e;
        }
    }
    
    /** read the history file
     * @throws IOException if the file doesn't exist
     * @return The history file in a String
     */    
    public String read() throws IOException {
        String s="";
        char[] buffer = new char[512];
        try {
            while (fileReader.read(buffer)>=0) {
                String tmp = "";
                tmp = tmp.copyValueOf(buffer);
                s.concat(tmp);
            }
            fileReader.close();
        }catch(IOException e) {
            throw e;
        }
        return s;
    }
    
    /** Return the next line of the history file
     * @throws IOException Return an IOException in case of problem
     * @return Return the next line of the history file
     */    
    public String nextLine() throws IOException {
        String s="";
        try {
            int c;
            char ch;
            while ((c=fileReader.read())>=0) {
                offset++;
                ch=(char)c;
                s=s.concat(s.valueOf(ch));
                if (ch=='\n') {
                    break;
                }
            }
        }catch(IOException e) {
            throw e;
        }
        return s;
    }
    
    /** Return the next 100 lines after offset*100 lines of the game
     * @param offset The offset
     * @throws IOException Return an IOException in case of problem
     * @return A String containing the 100 lines after the offset*100 lines of the history file
     */    
    public String next100LinesFrom(int offset) throws IOException {
        if (simulations.SimulationParameters.debug) {
            System.out.println("Offset = "+offset);
        }
        try {
            fileReader = new FileReader(file);
        }catch(FileNotFoundException e) {
            throw e;
        }
        String s="";
        int nb=0;
        boolean eof=false;
        while (nb<=offset) {
            int nbl=0;
            s="";
            while (nbl<100) {
                String s2="";
                try {
                    s2=nextLine();
                }catch(IOException e) {
                    throw e;
                }
                s=s.concat(s2);
                nbl++;
            }
            nb++;
        }
        //System.out.println(s);
        return s;
    }
    
    /** Return the next Ko of the file
     * @throws IOException Return an IOException in case of problem
     * @return Return the next Ko of the file
     */    
    public String nextKo() throws IOException {
        if (simulations.SimulationParameters.debug) {
            System.out.println("Offset = "+offset);
        }
        String s="";
        int nbch=0;
        char[] buffer = new char[1024];
        try {
            nbch=fileReader.read(buffer,0,1024);
            s = s.copyValueOf(buffer);
        }catch(IOException e) {
            throw e;
        }
        if (nbch<=0) {
            return "";
        }
        // continue until \n
        offset+=nbch;
        String s2="";
        try {
            int c;
            char ch;
            while ((c=fileReader.read())>=0) {
                offset++;
                ch=(char)c;
                s2=s2.concat(s2.valueOf(ch));
                if (ch=='\n') {
                    break;
                }
            }
        }catch(IOException e) {
            throw e;
        }
        //System.out.println(s);
        return s.concat(s2);
    }
    
    /** close the history file
     * @throws IOException Return an IOException in case of problem
     */    
    public void close() throws IOException {
        try {
            fileReader.close();
        }catch(IOException e) {
            throw e;
        }
    }
    
    /** Getter for property file.
     * @return Value of property file.
     *
     */
    public java.io.File getFile() {
        return file;
    }
    
    /** Setter for property file.
     * @param file New value of property file.
     *
     */
    public void setFile(java.io.File file) {
        this.file = file;
    }
    
    /** Getter for property offset.
     * @return Value of property offset.
     *
     */
    public int getOffset() {
        return offset;
    }
    
    /** Setter for property offset.
     * @param offset New value of property offset.
     *
     */
    public void setOffset(int offset) {
        this.offset = offset;
    }
    
}
