/*
 * HistoryWriter.java
 *
 * Created on 29 janvier 2004, 22:35
 */

package simulations.core.history;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import java.util.GregorianCalendar;
import java.util.Calendar;

/** This class describes the writer of a history file
 * @author tom
 */
public class HistoryWriter {
    
    private File file;
    
    private FileWriter fileWriter;
    
    private int traceLevel;
    
    /** Creates a new instance of HistoryWriter
     * @param traceLevel The trace level
     * @param file The history file
     * @throws IOException if an IO error occurs
     */
    public HistoryWriter(File file,int traceLevel) throws IOException {
        this.file = file;
        file.delete();
        try {
            file.createNewFile();
            if (simulations.SimulationParameters.debug) {
                System.out.println("create file : "+file.getAbsolutePath());
            }
            fileWriter = new FileWriter(file);
        }catch(IOException e) {
            throw e;
        }
        this.traceLevel=traceLevel;
    }
    
    /** Write a string in the history file with the fileWriter
     * @param text The text to write
     * @throws IOException if an IO error occurs
     */    
    public void write(String text) throws IOException {
        try {
            fileWriter.write(text);
        }catch(IOException e) {
            throw e;
        }
    }
    
    /** close the fileWriter
     * @throws IOException if an IO error occurs
     */    
    public void close() throws IOException {
        this.write("----------------------------------------------\n");
        GregorianCalendar date = new GregorianCalendar();
        String s = date.get(Calendar.DAY_OF_MONTH)+"/"+(date.get(Calendar.MONTH)+1)+"/"+
        date.get(Calendar.YEAR)+" "+date.get(Calendar.HOUR_OF_DAY)+"h"+date.get(Calendar.MINUTE)+
        "m"+date.get(Calendar.SECOND)+"s"+date.get(Calendar.MILLISECOND)+"ms";
        s+=" - End of the game\n";
        this.write(s);
        try {
            fileWriter.close();
        }catch(IOException e) {
            throw e;
        }
    }
    
    /** Getter for property file.
     * @return Value of property file.
     *
     */
    public java.io.File getFile() {
        return file;
    }
    
    /** Setter for property file.
     * @param file New value of property file.
     *
     */
    public void setFile(java.io.File file) {
        this.file = file;
    }
    
    /** Getter for property traceLevel.
     * @return Value of property traceLevel.
     *
     */
    public int getTraceLevel() {
        return traceLevel;
    }
    
    /** Setter for property traceLevel.
     * @param traceLevel New value of property traceLevel.
     *
     */
    public void setTraceLevel(int traceLevel) {
        this.traceLevel = traceLevel;
    }
    
}
