/*
 * PlayerStat.java
 *
 * Created on April 13, 2004, 2:14 AM
 */

package simulations.core.history;

import simulations.core.Player;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class memorize all the statistics of a player
 * @author  tom
 */
public class PlayersStats {
    
    /** The player **/
    private Player player;
    
    /** The list of statistics of attacked player **/
    private ArrayList attackedStatsList;
    
    /** The list of statistics of supported attack **/
    private ArrayList supportsStatsList;
    
    /** The list of statistics of asking support **/
    private ArrayList askingStatsList;
    
    
    /** Creates a new instance of PlayerStat
     * @param player The player to memorize statistics
     */
    public PlayersStats(Player player) {
        this.player=player;
        attackedStatsList = new ArrayList();
        askingStatsList = new ArrayList();
        supportsStatsList = new ArrayList();
    }
    
    /** Getter for property askingStatsList.
     * @return Value of property askingStatsList.
     *
     */
    public ArrayList getAskingStatsList() {
        return askingStatsList;
    }
    
    /** Setter for property askingStatsList.
     * @param askingStatsList New value of property askingStatsList.
     *
     */
    public void setAskingStatsList(ArrayList askingStatsList) {
        this.askingStatsList = askingStatsList;
    }
    
    /** Getter for property attackedStatsList.
     * @return Value of property attackedStatsList.
     *
     */
    public ArrayList getAttackedStatsList() {
        return attackedStatsList;
    }
    
    /** Setter for property attackedStatsList.
     * @param attackedStatsList New value of property attackedStatsList.
     *
     */
    public void setAttackedStatsList(ArrayList attackedStatsList) {
        this.attackedStatsList = attackedStatsList;
    }
    
    /** Getter for property player.
     * @return Value of property player.
     *
     */
    public simulations.core.Player getPlayer() {
        return player;
    }
    
    /** Setter for property player.
     * @param player New value of property player.
     *
     */
    public void setPlayer(simulations.core.Player player) {
        this.player = player;
    }
    
    /** Getter for property supportsStatsList.
     * @return Value of property supportsStatsList.
     *
     */
    public ArrayList getSupportsStatsList() {
        return supportsStatsList;
    }
    
    /** Setter for property supportsStatsList.
     * @param supportsStatsList New value of property supportsStatsList.
     *
     */
    public void setSupportsStatsList(ArrayList supportsStatsList) {
        this.supportsStatsList = supportsStatsList;
    }
    
    /** update the data with a playerTurn 
     * @param pt The playerTurn to use to update
     **/
    public void update(PlayerTurn pt) {
        addAttack(pt.getAttackedPlayer());
        Iterator it = pt.getPlayersAcceptingSupport().iterator();
        while (it.hasNext()) {
            Player p = (Player)it.next();
            addAsking(p);
            addSupport(p);
        }
        it = pt.getPlayersRefusingSupport().iterator();
        while (it.hasNext()) {
            addAsking((Player)it.next());
        }
    }
    
    /** Add an attack of the player against another
     * @param attackedPlayer The attacked player
     **/
    public void addAttack(Player attackedPlayer) {
        PlayerStatElement elt = getAttackedPlayerElement(attackedPlayer);
        if (elt==null) {
            attackedStatsList.add(new PlayerStatElement(attackedPlayer, 1));
        } else {
            elt.setNb(elt.getNb()+1);
        }
    }
    
    /** Add an asking of support of the player to another
     * @param askedPlayer The asked player
     **/
    public void addAsking(Player askedPlayer) {
        PlayerStatElement elt = getAskingPlayerElement(askedPlayer);
        if (elt==null) {
            askingStatsList.add(new PlayerStatElement(askedPlayer, 1));
        } else {
            elt.setNb(elt.getNb()+1);
        }
    }
    
    /** Add an acceptation of support of the player against another
     * @param attackedPlayer The attacked player
     **/
    public void addSupport(Player attackedPlayer) {
        PlayerStatElement elt = getSupportedPlayerElement(attackedPlayer);
        if (elt==null) {
            supportsStatsList.add(new PlayerStatElement(attackedPlayer, 1));
        } else {
            elt.setNb(elt.getNb()+1);
        }
    }
    
    /** Return the element storing stats about the attacked player
     * @param attackedPlayer attackedPlayer
     * @return Return the element storing stats about the attacked player
     **/
    public PlayerStatElement getAttackedPlayerElement(Player attackedPlayer) {
        Iterator it = attackedStatsList.iterator();
        while (it.hasNext()) {
            PlayerStatElement elt = (PlayerStatElement)it.next();
            if (elt.getPlayer().equals(attackedPlayer)) {
                return elt;
            }
        }
        return null;
    }
    
    /** Return the element storing stats about the asking player
     * @param askingPlayer The asking player
     * @return Return the element storing stats about the attacked player
     **/
    public PlayerStatElement getAskingPlayerElement(Player askingPlayer) {
        Iterator it = askingStatsList.iterator();
        while (it.hasNext()) {
            PlayerStatElement elt = (PlayerStatElement)it.next();
            if (elt.getPlayer().equals(askingPlayer)) {
                return elt;
            }
        }
        return null;
    }
    
    /** Return the element storing stats about the supporting player
     * @param attackedPlayer attackedPlayer
     * @return Return the element storing stats about the attacked player
     **/
    public PlayerStatElement getSupportedPlayerElement(Player attackedPlayer) {
        Iterator it = supportsStatsList.iterator();
        while (it.hasNext()) {
            PlayerStatElement elt = (PlayerStatElement)it.next();
            if (elt.getPlayer().equals(attackedPlayer)) {
                return elt;
            }
        }
        return null;
    }
    
}
