/*
 * AttackStrategy.java
 *
 * Created on April 17, 2004, 7:35 PM
 */

package simulations.core.strategy.attackStrategy;

import java.util.ArrayList;
import simulations.core.Player;
import simulations.core.history.Turn;
import java.io.Serializable;

/**
 * This abstract class describes haw to construct an attack strategy
 * @author  tom
 */
public abstract class AttackStrategy extends simulations.core.strategy.Strategy implements Serializable, Cloneable {
    
    /** True if the strategy is the negation **/
    protected boolean negative;
    
    /** True if the strategy is memory signifiant **/
    protected boolean memoryParam;
    
    /** Creates a new instance of AttackStrategy */
    public AttackStrategy() {
        super();
        negative=false;
    }
    
    /** return the player to attack
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     * @return The player to attack
     */    
    public abstract ArrayList getPlayerToAttack(Player player, ArrayList playerList);
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public abstract void updateLastTurn(Turn turn, Player player, ArrayList remainingPlayers);
    
    /** return the description of this attack strategy
     * @return return the description of this attack strategy
     **/
    public abstract String getDescription();

    /** overload of equals method
     * @param o The object to compare
     * @return Returns true if this and o are equals
     **/
    public boolean equals(Object o) {
        if (o instanceof AttackStrategy) {
            if (name.equals(((AttackStrategy)o).getName())) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Getter for property negative.
     * @return Value of property negative.
     */
    public boolean isNegative() {
        return negative;
    }
    
    /**
     * Setter for property negative.
     * @param negative New value of property negative.
     */
    public void setNegative(boolean negative) {
        this.negative = negative;
    }
    
    /**
     * Getter for property memoryParam.
     * @return Value of property memoryParam.
     */
    public boolean isMemoryParam() {
        return memoryParam;
    }
    
    /**
     * Setter for property memoryParam.
     * @param memoryParam New value of property memoryParam.
     */
    public void setMemoryParam(boolean memoryParam) {
        this.memoryParam = memoryParam;
    }
    
}
