/*
 * GenericComposedStrategy.java
 *
 * Created on April 18, 2004, 2:39 AM
 */

package simulations.core.strategy.completeStrategy;

import java.util.ArrayList;
import java.util.Iterator;

import java.io.Serializable;

import simulations.core.Player;
import simulations.core.history.Turn;
import simulations.core.exceptions.*;

import simulations.core.strategy.attackStrategy.AttackStrategy;
import simulations.core.strategy.askingSupportStrategy.AskingStrategy;
import simulations.core.strategy.decisionSupportStrategy.DecisionSupportStrategy;

/**
 * This class describes a generic CompsoedStrategy. We can add attack, asking and decision strategies.
 * This class is generic because to create new Strategies we just have to extends this class and
 * init the attack, asking and decision strategies.
 * @author  tom
 */
public class GenericComposedStrategy extends CompleteStrategy implements Serializable {
    
    /** Creates a new instance of GenericComposedStrategy */
    public GenericComposedStrategy() {
        super();
        name="GenericComposedStrategy";
    }
    
    /** Creates a new instance of GenericComposedStrategy
     * @param name The name of the strategy
     **/
    public GenericComposedStrategy(String name) {
        super();
        this.name=name;
    }
    
    /** return the description of this strategy
     * @return return the description of this strategy
     **/
    public String getDescription() {
        Iterator it = attackStrategyVector.iterator();
        String s = "";
        int cpt=1;
        while (it.hasNext()) {
            AttackStrategy strat = (AttackStrategy)it.next();
            s=s+cpt+" : ";
            s=s+strat.getDescription();
            cpt++;
        }
        it=askingStrategyVector.iterator();
        while (it.hasNext()) {
            AskingStrategy strat = (AskingStrategy)it.next();
            s=s+strat.getDescription();
        }
        it = decisionStrategyVector.iterator();
        while (it.hasNext()) {
            DecisionSupportStrategy strat = (DecisionSupportStrategy)it.next();
            s=s+strat.getDescription();
        }
        return s;
        
    }
    
    /** return the player to attack
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     * @return The player to attack
     * @throws ManyPlayerToAttackException if this method would return more that one player to attack
     * @throws NoPlayerToAttackException if this method would return no player to attack
     */    
    public simulations.core.Player getPlayerToAttack(simulations.core.Player player, java.util.ArrayList playerList) throws ManyPlayerToAttackException, NoPlayerToAttackException {
        boolean ok = false;
        ArrayList l = (ArrayList)playerList.clone();
        ArrayList lastList = playerList;
        Iterator it = attackStrategyVector.iterator();
        while (it.hasNext()) {
            AttackStrategy strat = (AttackStrategy)it.next();
            ArrayList l2 = strat.getPlayerToAttack(player, l);
            if (l2.size()>0) {
                lastList=l2;
            }
            if (l2.size()==1) {
                return (Player)l2.get(0);
            } else if (l2.size()==0) {
                l=lastList;
            } else {
                l=l2;
            }
        }
        if (l.size()==0) {
            throw new NoPlayerToAttackException();
        } else {
            throw new ManyPlayerToAttackException();
        }
    }
    
    /** Returns the list of players to ask support.
     * @return Returns the list of players to ask support.
     * @param attackedPlayer The attacked player
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     */    
    public java.util.ArrayList getPlayersToAskSupport(simulations.core.Player player, simulations.core.Player attackedPlayer, java.util.ArrayList playerList) {
        Iterator it = askingStrategyVector.iterator();
        ArrayList l = playerList;
        for(int i=0;i<askingStrategyVector.size();i++) {
            AskingStrategy askingStrat = (AskingStrategy)it.next();
            l = askingStrat.getPlayersToAskSupport(player, attackedPlayer, l);
        }
        return l;
    }
    
    /**
     * return true if the player will support the attack.
     * @return return true if the player will support the attack.
     * @param playerList The list of remaining players
     * @param player The owner of the strategy
     * @param attackingPlayer The attacking player
     * @param attackedPlayer The attacked player
     */    
    public boolean supportAttack(Player player, Player attackingPlayer, Player attackedPlayer, ArrayList playerList) {
        Iterator it = decisionStrategyVector.iterator();
        while (it.hasNext()) {
            DecisionSupportStrategy decisionStrategy = (DecisionSupportStrategy)it.next();
            if (!(decisionStrategy.supportAttack(player, attackingPlayer, attackedPlayer, playerList))) {
                if (simulations.SimulationParameters.debug) {
                    System.out.println(player.toString()+"refuse d'aider "+attackingPlayer.toString()+
                    " vs "+attackedPlayer.toString());
                }
                return false;
            }
        }
        if (simulations.SimulationParameters.debug) {
            System.out.println(player.toString()+"accepte d'aider "+attackingPlayer.toString()+
            " vs "+attackedPlayer.toString());
        }
        return true;
    }
    
    /**
     * This method is called by the judge after the decision of players to attack and to ask supports
     * to inform the strategy
     * @param player The player owner of this strategy
     * @param attacksList The list of attacks
     **/
    public void updateAttacksInfos(Player player, java.util.ArrayList attacksList) {
        Iterator it = decisionStrategyVector.iterator();
        while (it.hasNext()) {
            DecisionSupportStrategy strat = (DecisionSupportStrategy)it.next();
            strat.updateAttacksInfos(player,attacksList);
        }
    }
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public void updateLastTurn(simulations.core.history.Turn turn, simulations.core.Player player, java.util.ArrayList remainingPlayers) {
        Iterator it = attackStrategyVector.iterator();
        while (it.hasNext()) {
            AttackStrategy strat = (AttackStrategy)it.next();
            strat.updateLastTurn(turn, player, remainingPlayers);
        }
        it = askingStrategyVector.iterator();
        while (it.hasNext()) {
            AskingStrategy strat = (AskingStrategy)it.next();
            strat.updateLastTurn(turn, player, remainingPlayers);
        }
        it = decisionStrategyVector.iterator();
        while (it.hasNext()) {
            DecisionSupportStrategy strat = (DecisionSupportStrategy)it.next();
            strat.updateLastTurn(turn, player, remainingPlayers);
        }
    }
    
}
