/*
 * RandomStrategy.java
 *
 * Created on 22 janvier 2004, 00:21
 */

package simulations.core.strategy.completeStrategy;

import java.util.Random;
import java.util.ArrayList;

import simulations.core.Player;
import simulations.core.history.Turn;

import simulations.core.strategy.attackStrategy.AttackStrategy;
import simulations.core.strategy.attackStrategy.AttackRandomlyStrategy;
import simulations.core.strategy.askingSupportStrategy.AskingStrategy;
import simulations.core.strategy.askingSupportStrategy.RandomAskingStrategy;
import simulations.core.strategy.decisionSupportStrategy.RandomSupportStrategy;
import simulations.core.strategy.decisionSupportStrategy.DecisionSupportStrategy;

/** This class describes a simple strategy.
 * This strategy attack randomly a player and ask randomly support to some players
 * @author tom
 */
public class RandomStrategy extends CompleteStrategy {
    
    
    /** Creates a new instance of RandomStrategy
     */
    public RandomStrategy() {
        super();
        name="RandomStrategy";
        attackStrategyVector.addElement(new AttackRandomlyStrategy());
        askingStrategyVector.addElement(new RandomAskingStrategy());
        decisionStrategyVector.addElement(new RandomSupportStrategy());
    }

    /** return the player to attack
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     * @return The player to attack
     */    
    public Player getPlayerToAttack(Player player, ArrayList playerList) {
        // we remove the player to avoid he attack himself
        return (Player)((ArrayList)((AttackStrategy)attackStrategyVector.get(0)).getPlayerToAttack(player, playerList)).get(0);
    }
    
    /** Returns the list of players to ask support.
     * @return Returns the list of players to ask support.
     * @param attackedPlayer The attacked player
     * @param player The attacking player
     * @param playerList The list of <B>avalaible</B> player
     */    
    public ArrayList getPlayersToAskSupport(Player player, Player attackedPlayer, ArrayList playerList) {
        // we remove player to avoid he ask support to himself
        return ((AskingStrategy)askingStrategyVector.get(0)).getPlayersToAskSupport(player, attackedPlayer, playerList);
    }
    
    /**
     * return true if the player will support the attack.
     * @return return true if the player will support the attack.
     * @param playerList The remaining players list
     * @param player the owner of the strategy
     * @param attackingPlayer The attacking player
     * @param attackedPlayer The attacked player
     */    
    public boolean supportAttack(Player player, Player attackingPlayer, Player attackedPlayer, ArrayList playerList) {
        // 1 chance on 2 to accept
        return ((DecisionSupportStrategy)decisionStrategyVector.get(0)).supportAttack(player, attackingPlayer, attackedPlayer, playerList);
    }

    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public void updateLastTurn(Turn turn, Player player, ArrayList remainingPlayers) {
        
    }
    /** return the description of this strategy
     * @return return the description of this strategy
     **/
    public String getDescription() {
        return "Attack strategy : \n"+
        "I attack randomly a player in the remaining player list. \n"+
        "Asking support strategy : \n"+
        "I choose a number n randomly such that 0<n<number of remaining players \n"+
        "and I choose randomly n players in the list to ask support\n"+
        "Support decision strategy : \n"+
        "I accept to support an attack with a probability of 0.5 but I always \n"+
        "refuse to support an attack on myself.\n";
    }
    /**
     * This method is called by the judge after the decision of players to attack and to ask supports
     * to inform the strategy
     * @param player The owner of the strategy
     * @param attacksList The list of attacks
     */
    public void updateAttacksInfos(Player player, ArrayList attacksList) {
    }
    
}
