/*
 * DecisionSupportStrategy.java
 *
 * Created on April 17, 2004, 7:28 PM
 */

package simulations.core.strategy.decisionSupportStrategy;

import java.util.ArrayList;
import simulations.core.Player;
import simulations.core.history.Turn;
import java.io.Serializable;

/**
 * This abstract class describes haw to construct a decision strategy
 * @author  tom
 */
public abstract class DecisionSupportStrategy extends simulations.core.strategy.Strategy implements Cloneable, Serializable {
    
    /** True if the strategy is the negation **/
    protected boolean negative;
    
    /** True if the strategy is memory signifiant **/
    protected boolean memoryParam;
    
    /** Creates a new instance of DecisionSupportStrategy */
    public DecisionSupportStrategy() {
        super();
        negative=false;
    }
    
    /**
     * return true if the player will support the attack.
     * @return return true if the player will support the attack.
     * @param playerList The list of remaining players
     * @param player The owner of the strategy
     * @param attackingPlayer The attacking player
     * @param attackedPlayer The attacked player
     */    
    public abstract boolean supportAttack(Player player, Player attackingPlayer, Player attackedPlayer, ArrayList playerList);
    
    /**
     * This method is called by the judge after the decision of players to attack and to ask supports
     * to inform the strategy
     * @param player The owner of the strategy
     * @param attacksList The list of attacks
     **/
    public abstract void updateAttacksInfos(Player player, ArrayList attacksList);
    
    /** This method is called by the judge to inform players of the last turn
     * @param turn The turn
     * @param player The owner of the strategy
     * @param remainingPlayers The list of the remaining players
     */    
    public abstract void updateLastTurn(Turn turn, Player player, ArrayList remainingPlayers);
    
    /** return the description of this decision support strategy
     * @return return the description of this decision support strategy
     **/
    public abstract String getDescription();
    
    /** overload of equals method
     * @param o The object to compare
     * @return Returns true if this and o are equals
     **/
    public boolean equals(Object o) {
        if (o instanceof DecisionSupportStrategy) {
            if (name.equals(((DecisionSupportStrategy)o).getName())) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Getter for property negative.
     * @return Value of property negative.
     */
    public boolean isNegative() {
        return negative;
    }
    
    /**
     * Setter for property negative.
     * @param negative New value of property negative.
     */
    public void setNegative(boolean negative) {
        this.negative = negative;
    }
    
    /**
     * Getter for property memoryParam.
     * @return Value of property memoryParam.
     */
    public boolean isMemoryParam() {
        return memoryParam;
    }
    
    /**
     * Setter for property memoryParam.
     * @param memoryParam New value of property memoryParam.
     */
    public void setMemoryParam(boolean memoryParam) {
        this.memoryParam = memoryParam;
    }
    
}
