/*
 * MainFrame.java
 *
 * Created on 19 janvier 2004, 23:12
 */

package simulations.gui;

import simulations.core.Game;
import simulations.core.Judge;
import simulations.core.Player;
import simulations.xml.DocumentTracer;
import simulations.xml.DataChecker;
import simulations.core.history.HistoryReader;
import simulations.core.history.HistoryWriter;
import simulations.gui.coalitions.CoalitionView;
import simulations.gui.curves.CurvesDialog;
import simulations.core.strategy.PlayerIntElement;

import java.awt.Dimension;
import javax.swing.JFileChooser;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

import java.io.FileNotFoundException;

/** This class is the main frame of the game.
 * @author tom
 */
public class MainFrame extends javax.swing.JFrame {
    
    /** The game view panel **/
    private GameView gameView;
    
    /** The graph curves **/
    private CurvesDialog curvesDialog;
    
    /** The thread containing the curves dialog **/
    private Thread thread2;
    
    /** Creates new form MainFrame */
    public MainFrame() {
        initComponents();
        this.setIconImage(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/game.png")).getImage());
        //Dimension screenDimension =
        this.setSize(800, 600);
        gameView = new GameView();
        curvesDialog=null;
        getContentPane().add(gameView, java.awt.BorderLayout.CENTER);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        toolBar = new javax.swing.JToolBar();
        newButton = new javax.swing.JButton();
        loadFromXMLButton = new javax.swing.JButton();
        newShellLauncherButton = new javax.swing.JButton();
        playButton = new javax.swing.JButton();
        playStepByStepButton = new javax.swing.JButton();
        historyButton = new javax.swing.JButton();
        coalitionHistoryButton = new javax.swing.JButton();
        graphButton = new javax.swing.JButton();
        coalitionButton = new javax.swing.JToggleButton();
        threadGraphButton = new javax.swing.JToggleButton();
        statisticsButton = new javax.swing.JButton();
        coalitionCurvesButton = new javax.swing.JButton();
        coalitionRankButton = new javax.swing.JButton();
        jToolBar1 = new javax.swing.JToolBar();
        adminButton = new javax.swing.JButton();
        jToolBar2 = new javax.swing.JToolBar();
        editStrategiesHTMLButton = new javax.swing.JButton();
        editPlayersRankingHTMLButton = new javax.swing.JButton();
        editSummaryTestHTMLButton = new javax.swing.JButton();
        databaseButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        currentTurnLabel = new javax.swing.JLabel();
        nextTurnButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        newGameMenuItem = new javax.swing.JMenuItem();
        newXMLGameMenuItem = new javax.swing.JMenuItem();
        newLauncherMenuItem = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        gameMenu = new javax.swing.JMenu();
        playMenuItem = new javax.swing.JMenuItem();
        playStepByStepMenuItem = new javax.swing.JMenuItem();
        historyMenuItem = new javax.swing.JMenuItem();
        graphMenuItem = new javax.swing.JMenuItem();
        coalitionCheckBoxMenuItem = new javax.swing.JCheckBoxMenuItem();
        statisticsMenuItem = new javax.swing.JMenuItem();
        coalitionCurvesMenuItem = new javax.swing.JMenuItem();
        coalitionRankingMenuItem = new javax.swing.JMenuItem();
        coalitionHistoryMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        helpMenuItem = new javax.swing.JMenuItem();
        aboutMenuItem = new javax.swing.JMenuItem();

        setTitle("n players game: simulations");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        toolBar.setBorder(new javax.swing.border.EtchedBorder());
        newButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/new.png")));
        newButton.setToolTipText("New game...");
        newButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        newButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newButtonActionPerformed(evt);
            }
        });

        toolBar.add(newButton);

        loadFromXMLButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        loadFromXMLButton.setToolTipText("Open a XML file");
        loadFromXMLButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        loadFromXMLButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadFromXMLButtonActionPerformed(evt);
            }
        });

        toolBar.add(loadFromXMLButton);

        newShellLauncherButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/terminal.png")));
        newShellLauncherButton.setToolTipText("Edit a launching script");
        newShellLauncherButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        newShellLauncherButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newShellLauncherButtonActionPerformed(evt);
            }
        });

        toolBar.add(newShellLauncherButton);

        playButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/play.png")));
        playButton.setToolTipText("Play game");
        playButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        playButton.setEnabled(false);
        playButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playButtonActionPerformed(evt);
            }
        });

        toolBar.add(playButton);

        playStepByStepButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/player_end.png")));
        playStepByStepButton.setToolTipText("Play game step by step");
        playStepByStepButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        playStepByStepButton.setEnabled(false);
        playStepByStepButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playStepByStepButtonActionPerformed(evt);
            }
        });

        toolBar.add(playStepByStepButton);

        historyButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/edit.png")));
        historyButton.setToolTipText("See history");
        historyButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        historyButton.setEnabled(false);
        historyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyButtonActionPerformed(evt);
            }
        });

        toolBar.add(historyButton);

        coalitionHistoryButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/edit.png")));
        coalitionHistoryButton.setToolTipText("Coalition history dialog");
        coalitionHistoryButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        coalitionHistoryButton.setEnabled(false);
        coalitionHistoryButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionHistoryButtonActionPerformed(evt);
            }
        });

        toolBar.add(coalitionHistoryButton);

        graphButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/chart.png")));
        graphButton.setToolTipText("See curves of the game");
        graphButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        graphButton.setEnabled(false);
        graphButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graphButtonActionPerformed(evt);
            }
        });

        toolBar.add(graphButton);

        coalitionButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/kuser.png")));
        coalitionButton.setToolTipText("Toggle this button to view the coalitions formation during the game");
        coalitionButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        coalitionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionButtonActionPerformed(evt);
            }
        });

        toolBar.add(coalitionButton);

        threadGraphButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/chart.png")));
        threadGraphButton.setToolTipText("Toggle this button to see graph while game");
        threadGraphButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        toolBar.add(threadGraphButton);

        statisticsButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/admin.png")));
        statisticsButton.setToolTipText("See the statistics");
        statisticsButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        statisticsButton.setEnabled(false);
        statisticsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                statisticsButtonActionPerformed(evt);
            }
        });

        toolBar.add(statisticsButton);

        coalitionCurvesButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/chart2.png")));
        coalitionCurvesButton.setToolTipText("See the coalitions curve");
        coalitionCurvesButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        coalitionCurvesButton.setEnabled(false);
        coalitionCurvesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionCurvesButtonActionPerformed(evt);
            }
        });

        toolBar.add(coalitionCurvesButton);

        coalitionRankButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/kcalc.png")));
        coalitionRankButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        coalitionRankButton.setEnabled(false);
        coalitionRankButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionRankButtonActionPerformed(evt);
            }
        });

        toolBar.add(coalitionRankButton);

        adminButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/terminal.png")));
        adminButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        adminButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                adminButtonActionPerformed(evt);
            }
        });

        jToolBar1.add(adminButton);

        toolBar.add(jToolBar1);

        editStrategiesHTMLButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/html.png")));
        editStrategiesHTMLButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        editStrategiesHTMLButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editStrategiesHTMLButtonActionPerformed(evt);
            }
        });

        jToolBar2.add(editStrategiesHTMLButton);

        editPlayersRankingHTMLButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/html.png")));
        editPlayersRankingHTMLButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        editPlayersRankingHTMLButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editPlayersRankingHTMLButtonActionPerformed(evt);
            }
        });

        jToolBar2.add(editPlayersRankingHTMLButton);

        editSummaryTestHTMLButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/html.png")));
        editSummaryTestHTMLButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        editSummaryTestHTMLButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editSummaryTestHTMLButtonActionPerformed(evt);
            }
        });

        jToolBar2.add(editSummaryTestHTMLButton);

        databaseButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/databases_section.png")));
        databaseButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        databaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                databaseButtonActionPerformed(evt);
            }
        });

        jToolBar2.add(databaseButton);

        toolBar.add(jToolBar2);

        getContentPane().add(toolBar, java.awt.BorderLayout.NORTH);

        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        currentTurnLabel.setText("Current turn : ");
        jPanel1.add(currentTurnLabel);

        nextTurnButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/forward.png")));
        nextTurnButton.setText("Next turn");
        nextTurnButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextTurnButtonActionPerformed(evt);
            }
        });

        jPanel1.add(nextTurnButton);

        stopButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/stop.png")));
        stopButton.setText("Stop");
        stopButton.setEnabled(false);
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        jPanel1.add(stopButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        fileMenu.setText("File");
        newGameMenuItem.setText("New Game");
        newGameMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newGameMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(newGameMenuItem);

        newXMLGameMenuItem.setText("Load from XML file");
        newXMLGameMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newXMLGameMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(newXMLGameMenuItem);

        newLauncherMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/terminal.png")));
        newLauncherMenuItem.setText("Shell games launcher");
        newLauncherMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                newLauncherMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(newLauncherMenuItem);

        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        gameMenu.setText("Game");
        playMenuItem.setText("Play game");
        playMenuItem.setEnabled(false);
        playMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(playMenuItem);

        playStepByStepMenuItem.setText("Play step by step");
        playStepByStepMenuItem.setEnabled(false);
        playStepByStepMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playStepByStepMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(playStepByStepMenuItem);

        historyMenuItem.setText("See history");
        historyMenuItem.setEnabled(false);
        historyMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(historyMenuItem);

        graphMenuItem.setText("Curves");
        graphMenuItem.setEnabled(false);
        graphMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graphMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(graphMenuItem);

        coalitionCheckBoxMenuItem.setText("Coalitions");
        coalitionCheckBoxMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionCheckBoxMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(coalitionCheckBoxMenuItem);

        statisticsMenuItem.setText("Game statistics");
        statisticsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                statisticsMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(statisticsMenuItem);

        coalitionCurvesMenuItem.setText("Coalition curves");
        coalitionCurvesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionCurvesMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(coalitionCurvesMenuItem);

        coalitionRankingMenuItem.setText("Coalition ranking");
        coalitionRankingMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionRankingMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(coalitionRankingMenuItem);

        coalitionHistoryMenuItem.setText("Item");
        coalitionHistoryMenuItem.setEnabled(false);
        coalitionHistoryMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                coalitionHistoryMenuItemActionPerformed(evt);
            }
        });

        gameMenu.add(coalitionHistoryMenuItem);

        menuBar.add(gameMenu);

        helpMenu.setText("?");
        helpMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/help.png")));
        helpMenuItem.setText("Help");
        helpMenu.add(helpMenuItem);

        aboutMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/info.png")));
        aboutMenuItem.setText("About");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });

        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        pack();
    }//GEN-END:initComponents

    private void editSummaryTestHTMLButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editSummaryTestHTMLButtonActionPerformed
        // Add your handling code here:
        GenerateResultsDialog dialog = new GenerateResultsDialog(this, true);
        dialog.setVisible(true);
    }//GEN-LAST:event_editSummaryTestHTMLButtonActionPerformed

    private void databaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_databaseButtonActionPerformed
        // Add your handling code here:
        simulations.database.gui.DatabaseFrame frame = new simulations.database.gui.DatabaseFrame();
        frame.setVisible(true);
    }//GEN-LAST:event_databaseButtonActionPerformed

    private void editPlayersRankingHTMLButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editPlayersRankingHTMLButtonActionPerformed
        // Add your handling code here:
        simulations.gui.EditPlayersRankingHTMLDialog dialog = new simulations.gui.EditPlayersRankingHTMLDialog(this, true);
        dialog.setVisible(true);
    }//GEN-LAST:event_editPlayersRankingHTMLButtonActionPerformed

    private void editStrategiesHTMLButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editStrategiesHTMLButtonActionPerformed
        // Add your handling code here:
        simulations.gui.EditHTMLPlayersStrategyDialog dialog = new simulations.gui.EditHTMLPlayersStrategyDialog(this,true);
        dialog.setVisible(true);
    }//GEN-LAST:event_editStrategiesHTMLButtonActionPerformed

    private void adminButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_adminButtonActionPerformed
        // TODO add your handling code here:
        ScriptsFrame frame = new ScriptsFrame();
        frame.setVisible(true);
    }//GEN-LAST:event_adminButtonActionPerformed

    private void coalitionHistoryMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionHistoryMenuItemActionPerformed
        // TODO add your handling code here:
        coalitionHistoryButtonActionPerformed(evt);
    }//GEN-LAST:event_coalitionHistoryMenuItemActionPerformed

    private void coalitionHistoryButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionHistoryButtonActionPerformed
        // TODO add your handling code here:
        simulations.gui.coalitions.CoalitionHistoryDialog dialog = new simulations.gui.coalitions.CoalitionHistoryDialog(this,true,gameView.getJudge().getCoalitionListList(),gameView.getJudge().getGame().getStringCoalitionDefinition());
        dialog.setVisible(true);
    }//GEN-LAST:event_coalitionHistoryButtonActionPerformed
    
    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
        // TODO add your handling code here:
        gameView.getJudge().endGame();
        nextTurnButton.setEnabled(false);
        gameView.getJudge().endGame();
        ClassementDialog dialog = new ClassementDialog(this,true,gameView.getJudge().getClassement());
        dialog.setVisible(true);
        historyButton.setEnabled(true);
        graphButton.setEnabled(true);
        graphMenuItem.setEnabled(true);
        historyMenuItem.setEnabled(true);
        statisticsButton.setEnabled(true);
        statisticsMenuItem.setEnabled(true);
        coalitionCurvesButton.setEnabled(true);
        coalitionCurvesMenuItem.setEnabled(true);
        coalitionRankButton.setEnabled(true);
        coalitionRankingMenuItem.setEnabled(true);
        playButton.setEnabled(false);
        playMenuItem.setEnabled(false);
        playStepByStepButton.setEnabled(false);
        playStepByStepMenuItem.setEnabled(false);
        stopButton.setEnabled(false);
    }//GEN-LAST:event_stopButtonActionPerformed
    
    private void coalitionRankingMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionRankingMenuItemActionPerformed
        // TODO add your handling code here:
        coalitionRankButtonActionPerformed(evt);
    }//GEN-LAST:event_coalitionRankingMenuItemActionPerformed
    
    private void coalitionRankButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionRankButtonActionPerformed
        // TODO add your handling code here:
        simulations.gui.coalitions.CoalitionsRankingDialog dialog = new simulations.gui.coalitions.CoalitionsRankingDialog(this,true,gameView.getJudge().getCoalitionsRanking(), gameView.getJudge().getGame().getStringCoalitionDefinition());
        dialog.setVisible(true);
    }//GEN-LAST:event_coalitionRankButtonActionPerformed
    
    private void coalitionCurvesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionCurvesButtonActionPerformed
        // TODO add your handling code here:
        coalitionCurvesMenuItemActionPerformed(evt);
    }//GEN-LAST:event_coalitionCurvesButtonActionPerformed
    
    private void coalitionCurvesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionCurvesMenuItemActionPerformed
        // TODO add your handling code here:
        simulations.gui.curves.GenericCurvesDialog dialog = new simulations.gui.curves.GenericCurvesDialog(this,true,
        gameView.getJudge().coalitionListToData(gameView.getJudge().getCoalitionListList()),
        "Coalition curve");
        dialog.setVisible(true);
    }//GEN-LAST:event_coalitionCurvesMenuItemActionPerformed
    
    
    private void newLauncherMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newLauncherMenuItemActionPerformed
        // TODO add your handling code here:
        newShellLauncherButtonActionPerformed(evt);
    }//GEN-LAST:event_newLauncherMenuItemActionPerformed
    
    private void newShellLauncherButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newShellLauncherButtonActionPerformed
        // TODO add your handling code here:
        NewShellLauncherDialog dialog = new NewShellLauncherDialog(this,true);
        dialog.setVisible(true);
    }//GEN-LAST:event_newShellLauncherButtonActionPerformed
    
    private void loadFromXMLButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadFromXMLButtonActionPerformed
        // Add your handling code here:
        newXMLGameMenuItemActionPerformed(evt);
    }//GEN-LAST:event_loadFromXMLButtonActionPerformed
    
    private void newXMLGameMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newXMLGameMenuItemActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter("xml");
        filter.setDescription("XML file");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            if (simulations.SimulationParameters.debug) {
                System.out.println(fileName);
            }
            DocumentTracer parser = new DocumentTracer(fileName);
            DataChecker checker = new DataChecker();
            if (!(checker.checkName((ArrayList)parser.getPlayerList().clone()))) {
                MsgBox msgBox = new MsgBox(this,true,"All players must have a different name","",false);
                msgBox.setVisible(true);
                return;
            }
            gameView.getJudge().setGame(new simulations.core.Game(
            parser.getPlayerList(), parser.getSeed(),parser.getLoosingPoints(),parser.getWinnerPoints(),
            parser.getCoalitionPoints(),parser.isProportional(),parser.getTraceLevel(),parser.getHistoryFile(),
            parser.getGraphFile(),parser.getCoalitionDefinition(),false,true));
            if (simulations.SimulationParameters.debug) {
                System.out.println("trace level = "+gameView.getJudge().getGame().getTraceLevel());
            }
            gameView.initJList();
            playButton.setEnabled(true);
            playMenuItem.setEnabled(true);
            graphButton.setEnabled(false);
            graphMenuItem.setEnabled(false);
            statisticsButton.setEnabled(false);
            statisticsMenuItem.setEnabled(false);
            playStepByStepButton.setEnabled(true);
            playStepByStepMenuItem.setEnabled(true);
            historyButton.setEnabled(false);
            historyMenuItem.setEnabled(false);
            coalitionCheckBoxMenuItem.setEnabled(false);
            coalitionCurvesButton.setEnabled(false);
            coalitionCurvesMenuItem.setEnabled(false);
            coalitionRankButton.setEnabled(false);
            coalitionRankingMenuItem.setEnabled(false);
            coalitionHistoryButton.setEnabled(false);
            coalitionHistoryMenuItem.setEnabled(false);
            curvesDialog=null;
        }
    }//GEN-LAST:event_newXMLGameMenuItemActionPerformed
    
    private void nextTurnButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextTurnButtonActionPerformed
        // Add your handling code here:
        if (gameView.getJudge().getGame().isFinish()) {
            nextTurnButton.setEnabled(false);
            gameView.getJudge().endGame();
            ClassementDialog dialog = new ClassementDialog(this,true,gameView.getJudge().getClassement());
            dialog.setVisible(true);
            historyButton.setEnabled(true);
            graphButton.setEnabled(true);
            graphMenuItem.setEnabled(true);
            historyMenuItem.setEnabled(true);
            statisticsButton.setEnabled(true);
            statisticsMenuItem.setEnabled(true);
            coalitionCurvesButton.setEnabled(true);
            coalitionCurvesMenuItem.setEnabled(true);
            coalitionRankButton.setEnabled(true);
            coalitionRankingMenuItem.setEnabled(true);
            coalitionHistoryButton.setEnabled(true);
            coalitionHistoryMenuItem.setEnabled(true);
            playButton.setEnabled(false);
            playMenuItem.setEnabled(false);
            playStepByStepButton.setEnabled(false);
            playStepByStepMenuItem.setEnabled(false);
            stopButton.setEnabled(false);
            return;
        }
        if (gameView.getJudge().nextGameTurn()) {
            nextTurnButton.setEnabled(false);
            gameView.getJudge().endGame();
            MsgBox msgBox = new MsgBox(this,true,"Game stopped because the three last","turn are equals",false);
            msgBox.setVisible(true);
            ClassementDialog dialog = new ClassementDialog(this,true,gameView.getJudge().getClassement());
            dialog.setVisible(true);
            historyButton.setEnabled(true);
            graphButton.setEnabled(true);
            graphMenuItem.setEnabled(true);
            historyMenuItem.setEnabled(true);
            statisticsButton.setEnabled(true);
            statisticsMenuItem.setEnabled(true);
            coalitionCurvesButton.setEnabled(true);
            coalitionCurvesMenuItem.setEnabled(true);
            coalitionRankButton.setEnabled(true);
            coalitionRankingMenuItem.setEnabled(true);
            coalitionHistoryButton.setEnabled(true);
            coalitionHistoryMenuItem.setEnabled(true);
            playButton.setEnabled(false);
            playMenuItem.setEnabled(false);
            playStepByStepButton.setEnabled(false);
            playStepByStepMenuItem.setEnabled(false);
            stopButton.setEnabled(false);
            return;
        }
        currentTurnLabel.setText("Current turn : "+gameView.getJudge().getCurrentTurn());
        gameView.initJList();
        if (threadGraphButton.isSelected()) {
            if (gameView.getJudge().getCurrentTurn()==2) {
                ArrayList l = new ArrayList();
                for(int i=0;i<gameView.getJudge().getClassement().size();i++) {
                    PlayerIntElement elt = (PlayerIntElement)gameView.getJudge().getClassement().get(i);
                    l.add(new PlayerIntElement(elt.getPlayer(),elt.getNb()));
                }
                for(int i=0;i<gameView.getJudge().getGame().getRemainingPlayer().size();i++) {
                    Player p = (Player)gameView.getJudge().getGame().getRemainingPlayer().get(i);
                    if (!(containsPlayer(p,l))) {
                        l.add(new PlayerIntElement(p,gameView.getJudge().getCurrentTurn()));
                    }
                }
                curvesDialog = new CurvesDialog(null, false,l,gameView.getJudge().getPlayerListList());
                thread2 = new Thread(curvesDialog);
                thread2.start();
                if (coalitionButton.isSelected()) {
                    this.setSize(300, 600);
                    curvesDialog.setSize(800, 700);
                    curvesDialog.setLocation(300, 0);
                } else {
                    this.setSize(300, 600);
                    curvesDialog.setSize(800, 700);
                    curvesDialog.setLocation(300, 0);
                }
            } else if (gameView.getJudge().getCurrentTurn()>2) {
                ArrayList l = new ArrayList();
                for(int i=0;i<gameView.getJudge().getClassement().size();i++) {
                    PlayerIntElement elt = (PlayerIntElement)gameView.getJudge().getClassement().get(i);
                    l.add(new PlayerIntElement(elt.getPlayer(),elt.getNb()));
                }
                for(int i=0;i<gameView.getJudge().getGame().getRemainingPlayer().size();i++) {
                    Player p = (Player)gameView.getJudge().getGame().getRemainingPlayer().get(i);
                    if (!(containsPlayer(p,l))) {
                        l.add(new PlayerIntElement(p,gameView.getJudge().getCurrentTurn()));
                    }
                }
                curvesDialog.repaintGraph(l, gameView.getJudge().getPlayerListList());
            }
        }
    }//GEN-LAST:event_nextTurnButtonActionPerformed
    
    /** This method returns true if the player p is already in a PlayerIntElement list
     * @return true if the player p is already in the list false otherwise
     **/
    private boolean containsPlayer(Player p, ArrayList l) {
        for(int i=0;i<l.size();i++) {
            if (((PlayerIntElement)l.get(i)).getPlayer().equals(p)) {
                return true;
            }
        }
        return false;
    }
    
    private void statisticsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_statisticsButtonActionPerformed
        // Add your handling code here:
        statisticsMenuItemActionPerformed(evt);
    }//GEN-LAST:event_statisticsButtonActionPerformed
    
    private void statisticsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_statisticsMenuItemActionPerformed
        // Add your handling code here:
        StatisticsDialog dialog = new StatisticsDialog(this, false, gameView.getJudge().getStatisticsList());
        dialog.setVisible(true);
    }//GEN-LAST:event_statisticsMenuItemActionPerformed
    
    /** This method id called on a click on the coalition button
     * @param evt The event
     **/
    private void coalitionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionButtonActionPerformed
        // Add your handling code here:
        if (coalitionButton.isSelected()) {
            coalitionCheckBoxMenuItem.setSelected(true);
        } else {
            coalitionCheckBoxMenuItem.setSelected(false);
        }
    }//GEN-LAST:event_coalitionButtonActionPerformed
    
    /** This method id called on a click on the coalition check box menu item
     * @param evt The event
     **/
    private void coalitionCheckBoxMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_coalitionCheckBoxMenuItemActionPerformed
        // Add your handling code here:
        if (coalitionCheckBoxMenuItem.isSelected()) {
            coalitionButton.setSelected(true);
        } else {
            coalitionButton.setSelected(false);
        }
    }//GEN-LAST:event_coalitionCheckBoxMenuItemActionPerformed
    
    /** This method id called on a click on the graph button
     * @param evt The event
     **/
    private void graphButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_graphButtonActionPerformed
        // Add your handling code here:
        graphMenuItemActionPerformed(evt);
    }//GEN-LAST:event_graphButtonActionPerformed
    
    /** This method id called on a click on the graph menu item
     * @param evt The event
     **/
    private void graphMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_graphMenuItemActionPerformed
        // Add your handling code here:
        simulations.gui.curves.CurvesDialog dialog = new simulations.gui.curves.CurvesDialog(this, false, gameView.getJudge().getClassement(), (ArrayList)gameView.getJudge().getPlayerListList().clone());
        //dialog.setSize(800,600);
        dialog.setVisible(true);
    }//GEN-LAST:event_graphMenuItemActionPerformed
    
    /** This method id called on a click on the about menu item
     * @param evt The event
     **/
    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        // Add your handling code here:
        AboutDialog dialog = new AboutDialog(this,true);
        dialog.setVisible(true);
    }//GEN-LAST:event_aboutMenuItemActionPerformed
    
    /** This method id called on a click on the history menu item
     * @param evt The event
     **/
    private void historyMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyMenuItemActionPerformed
        // Add your handling code here:
        historyButtonActionPerformed(evt);
    }//GEN-LAST:event_historyMenuItemActionPerformed
    
    /** This method is called on a click on the history button
     * @param evt The event
     **/
    private void historyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyButtonActionPerformed
        // Add your handling code here:
        try {
            HistoryDialog dialog = new HistoryDialog(this, false, new HistoryReader(gameView.getJudge().getHistoryWriter().getFile()));
            dialog.setVisible(true);
        }catch(FileNotFoundException e) {
            System.out.println("Unable to read history file");
        }
    }//GEN-LAST:event_historyButtonActionPerformed
    
    /** This method id called on a click on the play step by step menu item
     * @param evt The event
     **/
    private void playStepByStepMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playStepByStepMenuItemActionPerformed
        // Add your handling code here:
        playStepByStepButtonActionPerformed(evt);
    }//GEN-LAST:event_playStepByStepMenuItemActionPerformed
    
    /** This method id called on a click on the play menu item
     * @param evt The event
     **/
    private void playMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playMenuItemActionPerformed
        // Add your handling code here:
        playButtonActionPerformed(evt);
    }//GEN-LAST:event_playMenuItemActionPerformed
    
    /** This method id called on a click on the play step by step button
     * @param evt The event
     **/
    private void playStepByStepButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playStepByStepButtonActionPerformed
        // Add your handling code here:
        CoalitionView coalitionView = null;
        Thread thread = null;
        if (coalitionButton.isSelected()) {
            coalitionView = new CoalitionView(this, false, gameView.getJudge().getGame().getPlayerList(),  gameView.getJudge().getGame().getPlayerList().size());
            thread = new Thread(coalitionView);
            thread.start();
            this.setSize(350, 600);
            coalitionView.setSize(350, 650);
            coalitionView.setLocation(350, 0);
        }
        gameView.setCoalitionView(coalitionView);
        gameView.getJudge().start();
        nextTurnButton.setEnabled(true);
        playStepByStepButton.setEnabled(false);
        playStepByStepMenuItem.setEnabled(false);
        playButton.setEnabled(false);
        stopButton.setEnabled(true);
        playStepByStepMenuItem.setEnabled(false);
        currentTurnLabel.setText("Current turn : "+gameView.getJudge().getCurrentTurn());
    }//GEN-LAST:event_playStepByStepButtonActionPerformed
    
    /** This method id called on a click on the play button
     * @param evt The event
     **/
    private void playButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playButtonActionPerformed
        // Add your handling code here:
        if (gameView.getJudge().getGame()!=null) {
            CoalitionView coalitionView = null;
            Thread thread = null;
            if (coalitionButton.isSelected()) {
                coalitionView = new CoalitionView(this, false, gameView.getJudge().getGame().getPlayerList(),  gameView.getJudge().getGame().getPlayerList().size());
                thread = new Thread(coalitionView);
                thread.start();
            }
            gameView.setCoalitionView(coalitionView);
            boolean b = gameView.playGame();
            gameView.initJList();
            //new MsgBox(this,true,"Game over", "",false).show();
            if (b) {
                MsgBox msgBox = new MsgBox(this,true,"Game stopped because the three last","turn are equals",false);
                msgBox.setVisible(true);
            }
            ClassementDialog dialog = new ClassementDialog(this,true,gameView.getJudge().getClassement());
            dialog.setVisible(true);
            historyButton.setEnabled(true);
            graphButton.setEnabled(true);
            graphMenuItem.setEnabled(true);
            historyMenuItem.setEnabled(true);
            statisticsButton.setEnabled(true);
            statisticsMenuItem.setEnabled(true);
            coalitionCheckBoxMenuItem.setEnabled(true);
            coalitionCurvesButton.setEnabled(true);
            coalitionCurvesMenuItem.setEnabled(true);
            coalitionRankButton.setEnabled(true);
            coalitionRankingMenuItem.setEnabled(true);
            coalitionHistoryButton.setEnabled(true);
            coalitionHistoryMenuItem.setEnabled(true);
            playButton.setEnabled(false);
            playMenuItem.setEnabled(false);
            playStepByStepButton.setEnabled(false);
            playStepByStepMenuItem.setEnabled(false);
        }
    }//GEN-LAST:event_playButtonActionPerformed
    
    /** This method id called on a click on the new game button
     * @param evt The event
     **/
    private void newButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newButtonActionPerformed
        // Add your handling code here:
        newGameMenuItemActionPerformed(evt);
    }//GEN-LAST:event_newButtonActionPerformed
    
    /** This method id called on a click on the new game menu item
     * @param evt The event
     **/
    private void newGameMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_newGameMenuItemActionPerformed
        // Add your handling code here:
        NewGameDialog dialog = new NewGameDialog(this,true);
        dialog.setVisible(true);
        if (dialog.getReturnStatus()==1) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("historyFile = "+dialog.getHistoryFile());
            }
            gameView.getJudge().setGame(new Game(dialog.getPlayerList(),dialog.getSeed(),dialog.getLoosingPoints(),
            dialog.getWinnerPoints(),dialog.getCoalitionPoints(),dialog.isProportional(),dialog.getTraceLevel(),
            dialog.getHistoryFile(),dialog.getGraphFile(),dialog.getCoalitionDefinition(),false,true));
            if (simulations.SimulationParameters.debug) {
                System.out.println("trace level = "+gameView.getJudge().getGame().getTraceLevel());
                System.out.println("Player list");
            }
            Iterator it = gameView.getJudge().getGame().getPlayerList().iterator();
            while (it.hasNext()) {
                Player p = (Player)it.next();
                if (simulations.SimulationParameters.debug) {
                    System.out.println(p.toString()+" : "+p.getStrategy().getClass().toString());
                    System.out.println(p.toString()+" : "+p.getStrategy().getRandomGenerator().toString());
                }
            }
            gameView.initJList();
            playButton.setEnabled(true);
            playMenuItem.setEnabled(true);
            graphButton.setEnabled(false);
            graphMenuItem.setEnabled(false);
            statisticsButton.setEnabled(false);
            statisticsMenuItem.setEnabled(false);
            playStepByStepButton.setEnabled(true);
            playStepByStepMenuItem.setEnabled(true);
            historyButton.setEnabled(false);
            historyMenuItem.setEnabled(false);
            coalitionCheckBoxMenuItem.setEnabled(false);
            coalitionCurvesButton.setEnabled(false);
            coalitionCurvesMenuItem.setEnabled(false);
            coalitionRankButton.setEnabled(false);
            coalitionRankingMenuItem.setEnabled(false);
            coalitionHistoryButton.setEnabled(false);
            coalitionHistoryMenuItem.setEnabled(false);
            curvesDialog=null;
        }
    }//GEN-LAST:event_newGameMenuItemActionPerformed
    
    /** This method id called on a click on the exit menu item
     * @param evt The event
     **/
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        // Add your handling code here:
        exitForm(null);
    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JButton adminButton;
    private javax.swing.JToggleButton coalitionButton;
    private javax.swing.JCheckBoxMenuItem coalitionCheckBoxMenuItem;
    private javax.swing.JButton coalitionCurvesButton;
    private javax.swing.JMenuItem coalitionCurvesMenuItem;
    private javax.swing.JButton coalitionHistoryButton;
    private javax.swing.JMenuItem coalitionHistoryMenuItem;
    private javax.swing.JButton coalitionRankButton;
    private javax.swing.JMenuItem coalitionRankingMenuItem;
    private javax.swing.JLabel currentTurnLabel;
    private javax.swing.JButton databaseButton;
    private javax.swing.JButton editPlayersRankingHTMLButton;
    private javax.swing.JButton editStrategiesHTMLButton;
    private javax.swing.JButton editSummaryTestHTMLButton;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu gameMenu;
    private javax.swing.JButton graphButton;
    private javax.swing.JMenuItem graphMenuItem;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem helpMenuItem;
    private javax.swing.JButton historyButton;
    private javax.swing.JMenuItem historyMenuItem;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JToolBar jToolBar2;
    private javax.swing.JButton loadFromXMLButton;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JButton newButton;
    private javax.swing.JMenuItem newGameMenuItem;
    private javax.swing.JMenuItem newLauncherMenuItem;
    private javax.swing.JButton newShellLauncherButton;
    private javax.swing.JMenuItem newXMLGameMenuItem;
    private javax.swing.JButton nextTurnButton;
    private javax.swing.JButton playButton;
    private javax.swing.JMenuItem playMenuItem;
    private javax.swing.JButton playStepByStepButton;
    private javax.swing.JMenuItem playStepByStepMenuItem;
    private javax.swing.JButton statisticsButton;
    private javax.swing.JMenuItem statisticsMenuItem;
    private javax.swing.JButton stopButton;
    private javax.swing.JToggleButton threadGraphButton;
    private javax.swing.JToolBar toolBar;
    // End of variables declaration//GEN-END:variables
    
}
