/*
 * NewShellLauncherDialog.java
 *
 * Created on May 8, 2004, 2:35 AM
 */

package simulations.gui;

import javax.swing.JFileChooser;
import javax.swing.SpinnerNumberModel;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Iterator;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Random;

import simulations.core.strategy.StrategyLoader;
import simulations.core.strategy.askingSupportStrategy.AskingStrategy;
import simulations.core.strategy.attackStrategy.AttackStrategy;
import simulations.core.strategy.decisionSupportStrategy.DecisionSupportStrategy;
import simulations.core.strategy.completeStrategy.CompleteStrategy;
import simulations.core.strategy.completeStrategy.GenericComposedStrategy;
import simulations.xml.XMLGameWriter;
import simulations.core.ScriptLauncher;
import simulations.core.Player;

/**
 * This class describes a dialog box to generate a script
 * to launch games
 * @author  tom
 */
public class NewShellLauncherDialog extends javax.swing.JDialog {
    
    /** The file to save the script **/
    private File saveFile;
    
    /** The xml file selected **/
    private File xmlFile;
    
    /** The simulations.jar file **/
    private File jarFile;
    
    /** The games list **/
    private ArrayList gameList;
    
    /** The list of all available attack strategies **/
    private ArrayList availableAttackStratList;
    
    /** The list of all available asking strategies **/
    private ArrayList availableAskingStratList;
    
    /** The list of all available decision strategies **/
    private ArrayList availableDecisionStratList;
    
    /** The list of all available complete strategies **/
    private ArrayList availableCompleteStratList;
    
    /** Creates new form NewShellLauncherDialog
     * @param parent The parent window
     * @param modal True if the dialog is modal
     **/
    public NewShellLauncherDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        initStrategies();
        gameList = new ArrayList();
        randomSeedSpinner.setModel(new SpinnerNumberModel(0,0,10000000,1));
        traceLevelSpinner.setModel(new SpinnerNumberModel(2,0,4,1));
        nbGamesSpinner.setModel(new SpinnerNumberModel(1,1,10000000,1));
        playerPointsSpinner.setModel(new SpinnerNumberModel(20,0,10000000,1));
        winnerPointsSpinner.setModel(new SpinnerNumberModel(1.0,0.0,1.0,0.1));
        memorySpinner.setModel(new SpinnerNumberModel(0,0,10000000,1));
        maxPlayerSpinner.setModel(new SpinnerNumberModel(10,3,10000000,1));
        attackListSizeSpinner.setModel(new SpinnerNumberModel(1,1,availableAttackStratList.size(),1));
        askingListSizeSpinner.setModel(new SpinnerNumberModel(1,1,availableAskingStratList.size(),1));
        decisionListSizeSpinner.setModel(new SpinnerNumberModel(1,1,availableDecisionStratList.size(),1));
        this.setLocationRelativeTo(parent);
    }
    
    private void initStrategies() {
        StrategyLoader loader = new StrategyLoader();
        availableAttackStratList = loader.getAllAttackStrategy();
        availableAskingStratList = loader.getAllAskingStrategy();
        availableDecisionStratList = loader.getAllDecisionStrategy();
        availableCompleteStratList = loader.getAllCompleteStrategy();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroup1 = new javax.swing.ButtonGroup();
        buttonGroup2 = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        saveButton = new javax.swing.JButton();
        saveLaunchButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        scriptFileTextField = new javax.swing.JTextField();
        scriptFileButton = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        shScriptRadioButton = new javax.swing.JRadioButton();
        dosScriptRadioButton = new javax.swing.JRadioButton();
        kshScriptRadioButton = new javax.swing.JRadioButton();
        jLabel12 = new javax.swing.JLabel();
        jarFolderTextField = new javax.swing.JTextField();
        jarFolderButton = new javax.swing.JButton();
        tabbedPane = new javax.swing.JTabbedPane();
        jPanel4 = new javax.swing.JPanel();
        xmlFileTextField = new javax.swing.JTextField();
        xmlGameFileButton = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        gamesJList = new javax.swing.JList();
        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        jPanel5 = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        nbGamesSpinner = new javax.swing.JSpinner();
        jLabel6 = new javax.swing.JLabel();
        maxPlayerSpinner = new javax.swing.JSpinner();
        saveGraphCheckBox = new javax.swing.JCheckBox();
        graphFolderTextField = new javax.swing.JTextField();
        graphFolderButton = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        playerPointsSpinner = new javax.swing.JSpinner();
        jLabel9 = new javax.swing.JLabel();
        historyFolderTextField = new javax.swing.JTextField();
        historyFolderButton = new javax.swing.JButton();
        randomSeedLabel = new javax.swing.JLabel();
        randomSeedSpinner = new javax.swing.JSpinner();
        timeCheckBox = new javax.swing.JCheckBox();
        jLabel10 = new javax.swing.JLabel();
        loosingPointsTextField = new javax.swing.JTextField();
        winnerPointsSpinner = new javax.swing.JSpinner();
        labelPlayerNumber1 = new javax.swing.JLabel();
        labelPlayerNumber2 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        activatedCheckBox = new javax.swing.JCheckBox();
        coalitionPointsTextField = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        traceLevelSpinner = new javax.swing.JSpinner();
        jLabel14 = new javax.swing.JLabel();
        memorySpinner = new javax.swing.JSpinner();
        randomlyMemoryCheckBox = new javax.swing.JCheckBox();
        jLabel15 = new javax.swing.JLabel();
        coalitionDef1RadioButton = new javax.swing.JRadioButton();
        coalitionDef2RadioButton = new javax.swing.JRadioButton();
        coalitionDef3RadioButton = new javax.swing.JRadioButton();
        jLabel16 = new javax.swing.JLabel();
        decisionListSizeSpinner = new javax.swing.JSpinner();
        askingListSizeSpinner = new javax.swing.JSpinner();
        jLabel17 = new javax.swing.JLabel();
        attackListSizeSpinner = new javax.swing.JSpinner();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        xmlFilesFolderTextField = new javax.swing.JTextField();
        xmlFilesGameButton = new javax.swing.JButton();
        coalitionDef4RadioButton = new javax.swing.JRadioButton();
        coalitionDef5RadioButton = new javax.swing.JRadioButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("New games script launcher ");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jLabel1.setFont(new java.awt.Font("Dialog", 1, 18));
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/terminal.png")));
        jLabel1.setText("New games script launcher");
        getContentPane().add(jLabel1, java.awt.BorderLayout.NORTH);

        saveButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/save.png")));
        saveButton.setText("Save");
        saveButton.setToolTipText("Save the script");
        saveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveButtonActionPerformed(evt);
            }
        });

        jPanel1.add(saveButton);

        saveLaunchButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/save.png")));
        saveLaunchButton.setText("Save & launch");
        saveLaunchButton.setToolTipText("Save & launch the script");
        saveLaunchButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveLaunchButtonActionPerformed(evt);
            }
        });

        jPanel1.add(saveLaunchButton);

        cancelButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/button_cancel.png")));
        cancelButton.setText("Cancel");
        cancelButton.setToolTipText("Close the dialog");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        jPanel1.add(cancelButton);

        getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel3.setLayout(new java.awt.GridBagLayout());

        jLabel2.setText("Script file : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(jLabel2, gridBagConstraints);

        scriptFileTextField.setColumns(25);
        scriptFileTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(scriptFileTextField, gridBagConstraints);

        scriptFileButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        scriptFileButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        scriptFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scriptFileButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(scriptFileButton, gridBagConstraints);

        jLabel3.setText("Script type : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(jLabel3, gridBagConstraints);

        shScriptRadioButton.setSelected(true);
        shScriptRadioButton.setText("Sh script");
        buttonGroup1.add(shScriptRadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(shScriptRadioButton, gridBagConstraints);

        dosScriptRadioButton.setText("DOS script");
        buttonGroup1.add(dosScriptRadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(dosScriptRadioButton, gridBagConstraints);

        kshScriptRadioButton.setText("Ksh script");
        buttonGroup1.add(kshScriptRadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(kshScriptRadioButton, gridBagConstraints);

        jLabel12.setText("Jar file : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(jLabel12, gridBagConstraints);

        jarFolderTextField.setColumns(25);
        jarFolderTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(jarFolderTextField, gridBagConstraints);

        jarFolderButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        jarFolderButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jarFolderButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jarFolderButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel3.add(jarFolderButton, gridBagConstraints);

        jPanel2.add(jPanel3, java.awt.BorderLayout.NORTH);

        jPanel4.setLayout(new java.awt.GridBagLayout());

        xmlFileTextField.setColumns(25);
        xmlFileTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        jPanel4.add(xmlFileTextField, gridBagConstraints);

        xmlGameFileButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        xmlGameFileButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        xmlGameFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xmlGameFileButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        jPanel4.add(xmlGameFileButton, gridBagConstraints);

        gamesJList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                gamesJListValueChanged(evt);
            }
        });

        jScrollPane1.setViewportView(gamesJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.gridheight = 4;
        jPanel4.add(jScrollPane1, gridBagConstraints);

        addButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/forward.png")));
        addButton.setText("Add");
        addButton.setEnabled(false);
        addButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        jPanel4.add(addButton, gridBagConstraints);

        removeButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/back.png")));
        removeButton.setText("Remove");
        removeButton.setEnabled(false);
        removeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        jPanel4.add(removeButton, gridBagConstraints);

        jLabel4.setText("XML game file  : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        jPanel4.add(jLabel4, gridBagConstraints);

        tabbedPane.addTab("From files", jPanel4);

        jPanel5.setLayout(new java.awt.GridBagLayout());

        jLabel5.setText("Number of games to generate : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel5, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(nbGamesSpinner, gridBagConstraints);

        jLabel6.setText("Max players : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel6, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(maxPlayerSpinner, gridBagConstraints);

        saveGraphCheckBox.setText("Save graph");
        saveGraphCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveGraphCheckBoxActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(saveGraphCheckBox, gridBagConstraints);

        graphFolderTextField.setColumns(15);
        graphFolderTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(graphFolderTextField, gridBagConstraints);

        graphFolderButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        graphFolderButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        graphFolderButton.setEnabled(false);
        graphFolderButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                graphFolderButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(graphFolderButton, gridBagConstraints);

        jLabel7.setText("Directory to save graph files : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 9;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel7, gridBagConstraints);

        jLabel8.setText("Player's points : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel8, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(playerPointsSpinner, gridBagConstraints);

        jLabel9.setText("Directory to save history files : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel9, gridBagConstraints);

        historyFolderTextField.setColumns(15);
        historyFolderTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(historyFolderTextField, gridBagConstraints);

        historyFolderButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        historyFolderButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        historyFolderButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                historyFolderButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(historyFolderButton, gridBagConstraints);

        randomSeedLabel.setText("Random seed : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(randomSeedLabel, gridBagConstraints);

        randomSeedSpinner.setToolTipText("Spin to enter the seed of random numbers generation");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(randomSeedSpinner, gridBagConstraints);

        timeCheckBox.setText("Time");
        timeCheckBox.setToolTipText("Check this box to set the seed with a fonction of time");
        timeCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                timeCheckBoxActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(timeCheckBox, gridBagConstraints);

        jLabel10.setText("Loosing points :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel10, gridBagConstraints);

        loosingPointsTextField.setColumns(10);
        loosingPointsTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        loosingPointsTextField.setText("1");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(loosingPointsTextField, gridBagConstraints);

        winnerPointsSpinner.setToolTipText("spin to enter the number of players");
        winnerPointsSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                winnerPointsSpinnerStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(winnerPointsSpinner, gridBagConstraints);

        labelPlayerNumber1.setText("Winner points :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(labelPlayerNumber1, gridBagConstraints);

        labelPlayerNumber2.setText("Coalition points :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(labelPlayerNumber2, gridBagConstraints);

        jLabel11.setText("Proportional side-payment :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 16;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel11, gridBagConstraints);

        activatedCheckBox.setText("activated");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 16;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(activatedCheckBox, gridBagConstraints);

        coalitionPointsTextField.setColumns(10);
        coalitionPointsTextField.setEditable(false);
        coalitionPointsTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        coalitionPointsTextField.setText("0");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(coalitionPointsTextField, gridBagConstraints);

        jLabel13.setText("Trace level for history : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel13, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(traceLevelSpinner, gridBagConstraints);

        jLabel14.setText("Memory fixed : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(jLabel14, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(memorySpinner, gridBagConstraints);

        randomlyMemoryCheckBox.setText("Randomly");
        randomlyMemoryCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                randomlyMemoryCheckBoxActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(randomlyMemoryCheckBox, gridBagConstraints);

        jLabel15.setText("Coalition definition :");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 17;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(jLabel15, gridBagConstraints);

        coalitionDef1RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef1RadioButton.setSelected(true);
        coalitionDef1RadioButton.setText("1. don't attack directlty each others");
        buttonGroup2.add(coalitionDef1RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 17;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(coalitionDef1RadioButton, gridBagConstraints);

        coalitionDef2RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef2RadioButton.setText("2. don't attack directly and indirectly each others ");
        buttonGroup2.add(coalitionDef2RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 18;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(coalitionDef2RadioButton, gridBagConstraints);

        coalitionDef3RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef3RadioButton.setText("3. don't attack directly and indirectly each others and support their attacks");
        buttonGroup2.add(coalitionDef3RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 19;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel5.add(coalitionDef3RadioButton, gridBagConstraints);

        jLabel16.setText("Max decision list size : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 15;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel16, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 15;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(decisionListSizeSpinner, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 14;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(askingListSizeSpinner, gridBagConstraints);

        jLabel17.setText("Max asking list size : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 14;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel17, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 13;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(attackListSizeSpinner, gridBagConstraints);

        jLabel18.setText("Max attack list size : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 13;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel18, gridBagConstraints);

        jLabel19.setText("Directory to save xml game files : ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(jLabel19, gridBagConstraints);

        xmlFilesFolderTextField.setColumns(15);
        xmlFilesFolderTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(xmlFilesFolderTextField, gridBagConstraints);

        xmlFilesGameButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/simulations/icons/folder.png")));
        xmlFilesGameButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        xmlFilesGameButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xmlFilesGameButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(xmlFilesGameButton, gridBagConstraints);

        coalitionDef4RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef4RadioButton.setText("4. 2 + attack same player");
        buttonGroup2.add(coalitionDef4RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 20;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(coalitionDef4RadioButton, gridBagConstraints);

        coalitionDef5RadioButton.setFont(new java.awt.Font("Dialog", 1, 10));
        coalitionDef5RadioButton.setText("5. 3 + attack same player");
        buttonGroup2.add(coalitionDef5RadioButton);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 21;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        jPanel5.add(coalitionDef5RadioButton, gridBagConstraints);

        tabbedPane.addTab("Randomly", jPanel5);

        jPanel2.add(tabbedPane, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents

    private void xmlFilesGameButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xmlFilesGameButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            xmlFilesFolderTextField.setText(chooser.getSelectedFile().getPath());
        }
    }//GEN-LAST:event_xmlFilesGameButtonActionPerformed

    private void saveLaunchButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveLaunchButtonActionPerformed
        // TODO add your handling code here:
        if (createScriptFile()) {
            // launch
            ScriptLauncher launcher = new ScriptLauncher(saveFile);
            Thread thread = new Thread(launcher);
            thread.start();
            closeDialog(null);
        }
    }//GEN-LAST:event_saveLaunchButtonActionPerformed
    
    private void winnerPointsSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_winnerPointsSpinnerStateChanged
        // TODO add your handling code here:
        Double tmp = null;
        try {
            tmp = (Double)winnerPointsSpinner.getValue();
        }catch(NumberFormatException e) {
            System.out.println(e.toString());
            return;
        }
        //System.out.println(tmp.doubleValue());
        Float tmp2=new Float(tmp.floatValue());
        if (simulations.SimulationParameters.debug) {
            System.out.println(tmp.floatValue());
            System.out.println(tmp2.floatValue());
        }
        if (tmp2.equals(new Float(0.0))) {
            coalitionPointsTextField.setText("1.0");
        } else if (tmp2.equals(new Float(0.1))) {
            coalitionPointsTextField.setText("0.9");
        } else if (tmp2.equals(new Float(0.2))) {
            coalitionPointsTextField.setText("0.8");
        } else if (tmp2.equals(new Float(0.3))) {
            coalitionPointsTextField.setText("0.7");
        } else if (tmp2.equals(new Float(0.4))) {
            coalitionPointsTextField.setText("0.6");
        } else if (tmp2.equals(new Float(0.5))) {
            coalitionPointsTextField.setText("0.5");
        } else if (tmp2.equals(new Float(0.6))) {
            coalitionPointsTextField.setText("0.4");
        } else if (tmp2.equals(new Float(0.7))) {
            coalitionPointsTextField.setText("0.3");
        } else if (tmp2.equals(new Float(0.8))) {
            coalitionPointsTextField.setText("0.2");
        } else if (tmp2.equals(new Float(0.9))) {
            coalitionPointsTextField.setText("0.1");
        } else {
            coalitionPointsTextField.setText("0.0");
        }
    }//GEN-LAST:event_winnerPointsSpinnerStateChanged
    
    private void saveGraphCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveGraphCheckBoxActionPerformed
        // TODO add your handling code here:
        if (saveGraphCheckBox.isSelected()) {
            graphFolderButton.setEnabled(true);
        } else {
            graphFolderButton.setEnabled(false);
        }
    }//GEN-LAST:event_saveGraphCheckBoxActionPerformed
    
    private void graphFolderButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_graphFolderButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            graphFolderTextField.setText(chooser.getSelectedFile().getPath());
        }
    }//GEN-LAST:event_graphFolderButtonActionPerformed
    
    private void historyFolderButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_historyFolderButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            historyFolderTextField.setText(chooser.getSelectedFile().getPath());
        }
    }//GEN-LAST:event_historyFolderButtonActionPerformed
    
    private void randomlyMemoryCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_randomlyMemoryCheckBoxActionPerformed
        // TODO add your handling code here:
        if (randomlyMemoryCheckBox.isSelected()) {
            memorySpinner.setEnabled(false);
        } else {
            memorySpinner.setEnabled(true);
        }
    }//GEN-LAST:event_randomlyMemoryCheckBoxActionPerformed
    
    private void timeCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_timeCheckBoxActionPerformed
        // TODO add your handling code here:
        if (timeCheckBox.isSelected()) {
            randomSeedSpinner.setEnabled(false);
        } else {
            randomSeedSpinner.setEnabled(true);
        }
    }//GEN-LAST:event_timeCheckBoxActionPerformed
    
    private void jarFolderButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jarFolderButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter();
        filter.addExtension("jar");
        filter.setDescription("Jar file");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            try {
                System.out.println(fileName);
            }catch(IndexOutOfBoundsException e) {
            }
            jarFile=new File(fileName);
            jarFolderTextField.setText(fileName);
        }
    }//GEN-LAST:event_jarFolderButtonActionPerformed

    /** create the script file
     * @return true if the script has been correctly created
     **/
    private boolean createScriptFile() {
        if (saveFile==null) {
            MsgBox msgBox = new MsgBox(null,true,"Please, select a file to save","",false);
            msgBox.setVisible(true);
            return false;
        }
        if (jarFile==null) {
            MsgBox msgBox = new MsgBox(null,true,"Please, select the jar file","",false);
            msgBox.setVisible(true);
            return false;
        }
        if (tabbedPane.getSelectedIndex()==0) {
            try {
                FileWriter writer = new FileWriter(saveFile);
                if (shScriptRadioButton.isSelected()) {
                    writer.write("#!/bin/sh\n");
                } else if (kshScriptRadioButton.isSelected()) {
                    writer.write("#!/usr/bin/ksh\n");
                }
                Iterator it = gameList.iterator();
                while (it.hasNext()) {
                    File f = (File)it.next();
                    if (jarFile==null) {
                        writer.write("java -jar simulations.jar "+f.getPath()+"\n");
                    } else {
                        writer.write("java -jar "+jarFile.getPath()+" "+f.getPath()+"\n");
                    }
                }
                writer.close();
            }catch(IOException e) {
                MsgBox msgBox = new MsgBox(null,true,"An error occured when saving file",saveFile.getPath(),false);
                msgBox.setVisible(true);
                System.out.println(e.toString());
                return false;
            }
        } else {
            try {
                FileWriter writer = new FileWriter(saveFile);
                if (shScriptRadioButton.isSelected()) {
                    writer.write("#/bin/sh\n");
                } else if (kshScriptRadioButton.isSelected()) {
                    writer.write("#/usr/bin/ksh\n");
                }
                long seed = 0;
                if (randomlyMemoryCheckBox.isSelected()) {
                    seed = new GregorianCalendar().get(Calendar.MILLISECOND);
                } else {
                    seed = ((Integer)randomSeedSpinner.getValue()).longValue();
                }
                Random randomGenerator = new Random(seed);
                Random random = new Random(new GregorianCalendar().get(Calendar.MILLISECOND));
                int traceLevel = ((Integer)traceLevelSpinner.getValue()).intValue();
                int maxPlayer = ((Integer)maxPlayerSpinner.getValue()).intValue();
                int playerPoints = ((Integer)playerPointsSpinner.getValue()).intValue();
                int nbGames = ((Integer)nbGamesSpinner.getValue()).intValue();
                int memory = ((Integer)memorySpinner.getValue()).intValue();
                int maxAttackList = ((Integer)attackListSizeSpinner.getValue()).intValue();
                int maxAskingList = ((Integer)askingListSizeSpinner.getValue()).intValue();
                int maxDecisionList = ((Integer)decisionListSizeSpinner.getValue()).intValue();
                double winnerPoint = ((Double)winnerPointsSpinner.getValue()).doubleValue();
                double coalitionPoint = new Double(coalitionPointsTextField.getText()).doubleValue();
                int loosingPoint = new Integer(loosingPointsTextField.getText()).intValue();
                int coalitionDefinition = 0;
                if (coalitionDef1RadioButton.isSelected()) {
                    coalitionDefinition = 0;
                } else if (coalitionDef2RadioButton.isSelected()) {
                    coalitionDefinition = 1;
                } else if (coalitionDef3RadioButton.isSelected()) {
                    coalitionDefinition = 2;
                } else if (coalitionDef4RadioButton.isSelected()) {
                    coalitionDefinition = 3;
                } else {
                    coalitionDefinition = 4;
                }
                // redo newInstance() on each strat!
                String gameFolder=saveFile.getParent();
                if (simulations.SimulationParameters.debug) {
                    System.out.println("game folder = "+gameFolder);
                }
                if (historyFolderTextField.getText().equals("")) {
                    MsgBox msgBox = new MsgBox(null,true,"Please, select a folder to save history files","",false);
                    msgBox.setVisible(true);
                    return false;
                }
                String historyFolder=historyFolderTextField.getText();
                if (saveGraphCheckBox.isSelected()) {
                    if (graphFolderTextField.getText().equals("")) {
                        MsgBox msgBox = new MsgBox(null,true,"Please, select a folder to save graph files","",false);
                        msgBox.setVisible(true);
                        return false;
                    }
                }
                String graphFolder=graphFolderTextField.getText();
                String xmlFilesFolder=xmlFilesFolderTextField.getText();
                if (simulations.SimulationParameters.debug) {
                    System.out.println(nbGames+" games to create");
                }
                for(int i=0;i<nbGames;i++) {
                    ArrayList playerList = new ArrayList();
                    int nbPlayer = random.nextInt(maxPlayer-3);
                    nbPlayer+=3;
                    if (simulations.SimulationParameters.debug) {
                        System.out.println("game#"+i+" : "+nbPlayer+" players");
                    }
                    for(int j=0;j<nbPlayer;j++) {
                        // construct player list
                        GenericComposedStrategy strat = new GenericComposedStrategy("Strategy"+j);
                        strat.setRandomGenerator(randomGenerator);
                        strat.setMemory(memory);
                        int nbAttackStrat = random.nextInt(maxAttackList);
                        nbAttackStrat++;
                        Vector attackList = new Vector();
                        int cpt=nbAttackStrat;
                        while (cpt>0) {
                            AttackStrategy attackStrat = (AttackStrategy)availableAttackStratList.get(random.nextInt(availableAttackStratList.size()));
                            AttackStrategy attackStrat2 = null;
                            try {
                                attackStrat2 = (AttackStrategy)attackStrat.getClass().newInstance();
                            }catch(InstantiationException e) {
                                return false;
                            }catch(IllegalAccessException e) {
                                return false;
                            }
                            attackStrat2.setMemory(memory);
                            if (random.nextFloat()>0.5) {
                                attackStrat2.setNegative(true);
                            } else {
                                attackStrat2.setNegative(false);
                            }
                            attackStrat2.setName(attackStrat.getName());
                            attackStrat2.setRandomGenerator(randomGenerator);
                            attackList.add(attackStrat2);
                            cpt--;
                        }
                        attackList.add(new simulations.core.strategy.attackStrategy.AttackRandomlyStrategy());
                        strat.setAttackStrategyVector(attackList);
                        
                        int nbAskingStrat = random.nextInt(maxAskingList);
                        nbAskingStrat++;
                        Vector askingList = new Vector();
                        cpt=nbAskingStrat;
                        while (cpt>0) {
                            AskingStrategy askingStrat = (AskingStrategy)availableAskingStratList.get(random.nextInt(availableAskingStratList.size()));
                            AskingStrategy askingStrat2 = null;
                            try {
                                askingStrat2 = (AskingStrategy)askingStrat.getClass().newInstance();
                            }catch(InstantiationException e) {
                                return false;
                            }catch(IllegalAccessException e) {
                                return false;
                            }
                            askingStrat2.setMemory(memory);
                            if (random.nextFloat()>0.5) {
                                askingStrat2.setNegative(true);
                            } else {
                                askingStrat2.setNegative(false);
                            }
                            askingStrat2.setName(askingStrat.getName());
                            askingStrat2.setRandomGenerator(randomGenerator);
                            askingList.add(askingStrat2);
                            cpt--;
                        }
                        strat.setAskingStrategyVector(askingList);
                        
                        int nbDecisionStrat = random.nextInt(maxDecisionList);
                        nbDecisionStrat++;
                        Vector decisionList = new Vector();
                        cpt=nbDecisionStrat;
                        while (cpt>0) {
                            DecisionSupportStrategy decisionStrat = (DecisionSupportStrategy)availableDecisionStratList.get(random.nextInt(availableDecisionStratList.size()));
                            DecisionSupportStrategy decisionStrat2 = null;
                            try {
                                decisionStrat2 = (DecisionSupportStrategy)decisionStrat.getClass().newInstance();
                            }catch(InstantiationException e) {
                                return false;
                            }catch(IllegalAccessException e) {
                                return false;
                            }
                            decisionStrat2.setMemory(memory);
                            if (random.nextFloat()>0.5) {
                                decisionStrat2.setNegative(true);
                            } else {
                                decisionStrat2.setNegative(false);
                            }
                            decisionStrat2.setName(decisionStrat.getName());
                            decisionStrat2.setRandomGenerator(randomGenerator);
                            decisionList.add(decisionStrat2);
                            cpt--;
                        }
                        strat.setDecisionStrategyVector(decisionList);
                        playerList.add(new Player("Player #"+j,playerPoints,strat));
                    }
                    String gameFileName = xmlFilesFolder+"/game"+i+".xml";
                    File gameFile = new File(gameFileName);
                    XMLGameWriter xmlWriter = new XMLGameWriter();
                    File graphFile = null;
                    if (saveGraphCheckBox.isSelected()) {
                        graphFile = new File(graphFolder+"/graph"+i+".png");
                    }
                    xmlWriter.create(gameFile, seed,  loosingPoint, (float)winnerPoint, (float)coalitionPoint,
                    traceLevel,coalitionDefinition, activatedCheckBox.isSelected(),
                    new File(historyFolder+"/history"+i+".txt"),
                    graphFile, playerList);
                    writer.write("java -jar "+jarFile.getPath()+" "+gameFile.getPath()+"\n");
                }
                writer.close();
            }catch(IOException e) {
                MsgBox msgBox = new MsgBox(null,true,"An error occured when saving file",saveFile.getPath(),false);
                msgBox.setVisible(true);
                System.out.println(e.toString());
                return false;
            }
        }
        return true;
    }
    
    private void saveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveButtonActionPerformed
        // TODO add your handling code here:
        if (createScriptFile()) {
            closeDialog(null);
        }
    }//GEN-LAST:event_saveButtonActionPerformed
    
    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
        // TODO add your handling code here:
        // remove the game
        if (gamesJList.getSelectedValue()!=null) {
            File f = (File)gamesJList.getSelectedValue();
            gameList.remove(f);
            gamesJList.setListData(gameList.toArray());
        }
    }//GEN-LAST:event_removeButtonActionPerformed
    
    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
        // TODO add your handling code here:
        if (xmlFile!=null) {
            gameList.add(xmlFile);
            gamesJList.setListData(gameList.toArray());
        }
    }//GEN-LAST:event_addButtonActionPerformed
    
    private void gamesJListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_gamesJListValueChanged
        // TODO add your handling code here:
        if (gamesJList.getSelectedValue()==null) {
            removeButton.setEnabled(false);
        } else {
            removeButton.setEnabled(true);
        }
    }//GEN-LAST:event_gamesJListValueChanged
    
    private void xmlGameFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xmlGameFileButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter();
        filter.addExtension("xml");
        filter.setDescription("XML file");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            xmlFile = new File(fileName);
            xmlFileTextField.setText(fileName);
            addButton.setEnabled(true);
        }
    }//GEN-LAST:event_xmlGameFileButtonActionPerformed
    
    private void scriptFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scriptFileButtonActionPerformed
        // TODO add your handling code here:
        JFileChooser chooser = new JFileChooser();
        StrategyFileFilter filter = new StrategyFileFilter();
        if (shScriptRadioButton.isSelected()) {
            filter.addExtension("sh");
            filter.setDescription("Sh file");
        } else if (kshScriptRadioButton.isSelected()) {
            filter.addExtension("ksh");
            filter.setDescription("Ksh file");
        } else {
            filter.addExtension("bat");
            filter.setDescription("Bat file");
        }
        chooser.setFileFilter(filter);
        int returnVal = chooser.showSaveDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            if (simulations.SimulationParameters.debug) {
                System.out.println("You chose to save this file: " +
                chooser.getSelectedFile().getName());
            }
            String fileName = chooser.getSelectedFile().getPath();
            try {
                if (shScriptRadioButton.isSelected()) {
                    System.out.println("ext = "+fileName.substring(fileName.length()-3,fileName.length()));
                    if (!((fileName.substring(fileName.length()-3,fileName.length())).equals(".sh"))) {
                        fileName=fileName+".sh";
                    }
                } else if (kshScriptRadioButton.isSelected()) {
                    System.out.println("ext = "+fileName.substring(fileName.length()-4,fileName.length()));
                    if (!((fileName.substring(fileName.length()-4,fileName.length())).equals(".ksh"))) {
                        fileName=fileName+".ksh";
                    }
                } else {
                    System.out.println("ext = "+fileName.substring(fileName.length()-4,fileName.length()));
                    if (!((fileName.substring(fileName.length()-4,fileName.length())).equals(".bat"))) {
                        fileName=fileName+".bat";
                    }
                }
                System.out.println(fileName);
            }catch(IndexOutOfBoundsException e) {
            }
            saveFile=new File(fileName);
            scriptFileTextField.setText(fileName);
        }
    }//GEN-LAST:event_scriptFileButtonActionPerformed
    
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:
        closeDialog(evt);
    }//GEN-LAST:event_formWindowClosing
    
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        // TODO add your handling code here:
        closeDialog(null);
    }//GEN-LAST:event_cancelButtonActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {
        setVisible(false);
        dispose();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox activatedCheckBox;
    private javax.swing.JButton addButton;
    private javax.swing.JSpinner askingListSizeSpinner;
    private javax.swing.JSpinner attackListSizeSpinner;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.ButtonGroup buttonGroup2;
    private javax.swing.JButton cancelButton;
    private javax.swing.JRadioButton coalitionDef1RadioButton;
    private javax.swing.JRadioButton coalitionDef2RadioButton;
    private javax.swing.JRadioButton coalitionDef3RadioButton;
    private javax.swing.JRadioButton coalitionDef4RadioButton;
    private javax.swing.JRadioButton coalitionDef5RadioButton;
    private javax.swing.JTextField coalitionPointsTextField;
    private javax.swing.JSpinner decisionListSizeSpinner;
    private javax.swing.JRadioButton dosScriptRadioButton;
    private javax.swing.JList gamesJList;
    private javax.swing.JButton graphFolderButton;
    private javax.swing.JTextField graphFolderTextField;
    private javax.swing.JButton historyFolderButton;
    private javax.swing.JTextField historyFolderTextField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton jarFolderButton;
    private javax.swing.JTextField jarFolderTextField;
    private javax.swing.JRadioButton kshScriptRadioButton;
    private javax.swing.JLabel labelPlayerNumber1;
    private javax.swing.JLabel labelPlayerNumber2;
    private javax.swing.JTextField loosingPointsTextField;
    private javax.swing.JSpinner maxPlayerSpinner;
    private javax.swing.JSpinner memorySpinner;
    private javax.swing.JSpinner nbGamesSpinner;
    private javax.swing.JSpinner playerPointsSpinner;
    private javax.swing.JLabel randomSeedLabel;
    private javax.swing.JSpinner randomSeedSpinner;
    private javax.swing.JCheckBox randomlyMemoryCheckBox;
    private javax.swing.JButton removeButton;
    private javax.swing.JButton saveButton;
    private javax.swing.JCheckBox saveGraphCheckBox;
    private javax.swing.JButton saveLaunchButton;
    private javax.swing.JButton scriptFileButton;
    private javax.swing.JTextField scriptFileTextField;
    private javax.swing.JRadioButton shScriptRadioButton;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JCheckBox timeCheckBox;
    private javax.swing.JSpinner traceLevelSpinner;
    private javax.swing.JSpinner winnerPointsSpinner;
    private javax.swing.JTextField xmlFileTextField;
    private javax.swing.JTextField xmlFilesFolderTextField;
    private javax.swing.JButton xmlFilesGameButton;
    private javax.swing.JButton xmlGameFileButton;
    // End of variables declaration//GEN-END:variables
    
}
