/*
 * GenericCurvesPanel.java
 *
 * Created on May 19, 2004, 2:56 AM
 */

package simulations.gui.curves;

import javax.swing.JPanel;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;

import java.util.ArrayList;
import java.util.Iterator;



/**
 * This class describes a generic panel to construst a curve with the data (double[])
 * @author  tom
 */
public class GenericCurvesPanel extends javax.swing.JPanel {
    
    /** The data to draw **/
    private double[] data;
    
    /** The image to store the curves **/
    private Image offscreen;
        
    /** The min value of history to draw **/
    private int minX;
    
    /** The min value of points to draw **/
    private int minY;
    
    /** The max value of history to draw **/
    private int maxX;
    
    /** The number of turns **/
    private int maxMaxX;
    
    /** The max value of points to draw **/
    private int maxY;
    
    /** The max points **/
    private int maxMaxY;
    
    /** The ratio for one turn **/
    private Double pxOneTurn;
    
    /** The ratio for one point **/
    private Double pxOnePoint;
    
    /** The current mouse coords **/
    private Point mouseCoord;
    
    /** True if the mouse is pressed **/
    private boolean pressed;
    
    /** True if the cross is to display **/
    private boolean cross;
    
    /** True if the grid is to display **/
    private boolean grid;
    
    /** True if we need to redraw curves **/
    private boolean repaintCurveNeeded;
    
    /** The mouse coords when the mouse is pressed **/
    private Point pressedCoord;
    
    /** The rectangle draw when zoom **/
    private Rectangle zoomRectangle;
    
    /**
     * Creates new form GenericCurvesPanel
     * @param data The data to draw
     */
    public GenericCurvesPanel(double[] data) {
        initComponents();
        this.data=data;
        maxMaxX=data.length;
        maxX=maxMaxX;
        minX=0;
        minY=0;
        maxMaxY=maxNumber();
        maxY=maxMaxY;
        zoomRectangle=new Rectangle();
        repaintCurveNeeded=true;
        grid=false;
        pressedCoord = new Point();
        mouseCoord = new Point();
        this.setBackground(new java.awt.Color(255, 255, 255));
        this.setSize(800,600);
        cross=true;
        pressed=false;
        this.addMouseMotionListener(new java.awt.event.MouseMotionListener() {
            public void mouseDragged(java.awt.event.MouseEvent e) {
                //System.out.println("Mouse dragged handler");
                pressed=true;
                mouseCoord=e.getPoint();
                repaint();
            }
            public void mouseMoved(java.awt.event.MouseEvent e) {
                //System.out.println("Mouse moved handler");
                mouseCoord=e.getPoint();
                repaint();
            }
        } );
        this.addMouseListener(new java.awt.event.MouseListener() {
            public void mouseReleased(java.awt.event.MouseEvent e) {
                //System.out.println("Mouse released handler");
                pressed=false;
                // zoom
                if ((zoomRectangle.getWidth()<50)||(zoomRectangle.getHeight()<50)) {
                    repaint();
                    return;
                }
                
                // found max min by rectangle
                /**System.out.println("Points rectangle :");
                System.out.println("("+(int)zoomRectangle.getX()+","+(int)zoomRectangle.getY()+")");
                System.out.println("("+(int)(zoomRectangle.getX()+zoomRectangle.getWidth())+
                ","+(int)(zoomRectangle.getY()+zoomRectangle.getHeight())+")");**/
                int tmpMinX = coordToTurn((int)zoomRectangle.getX());
                int tmpMaxX = coordToTurn((int)(zoomRectangle.getX()+zoomRectangle.getWidth()));
                int tmpMinY = coordToPoint((int)(zoomRectangle.getY()+zoomRectangle.getHeight()));
                int tmpMaxY = coordToPoint((int)zoomRectangle.getY());
                if ((tmpMaxX-tmpMinX)<=3) {
                    return;
                }
                if ((tmpMaxY-tmpMinY)<=3) {
                    return;
                }
                minX=tmpMinX;
                minY=tmpMinY;
                maxX=tmpMaxX;
                maxY=tmpMaxY;
                //System.out.println("minX = "+minX+" maxX = "+maxX+" minY = "+minY+" maxY = "+maxY);
                repaintCurveNeeded=true;
                repaint();
            }
            public void mouseClicked(java.awt.event.MouseEvent e) {
                
            }
            public void mouseEntered(java.awt.event.MouseEvent e) {
                
            }
            public void mouseExited(java.awt.event.MouseEvent e) {
                
            }
            public void mousePressed(java.awt.event.MouseEvent e) {
                //System.out.println("Mouse pressed handler");
                pressed=true;
                mouseCoord=e.getPoint();
                pressedCoord=new Point((int)(e.getPoint().getX()),(int)(e.getPoint().getY()));
                repaint();
            }
        });
        this.addComponentListener(new java.awt.event.ComponentListener() {
            public void componentHidden(java.awt.event.ComponentEvent e) {
                
            }
            public void componentMoved(java.awt.event.ComponentEvent e) {
                
            }
            public void componentResized(java.awt.event.ComponentEvent e) {
                repaintCurveNeeded=true;
                repaint();
            }
            public void componentShown(java.awt.event.ComponentEvent e) {
                
            }
            
        });
    }
    
    /** Returns the max Y to display
     * @return the max Y to display
     **/
    private int maxNumber() {
        double max = 0;
        for(int i=minX;i<maxX;i++) {
            if (data[i]>max) {
                max=data[i];
            }
        }
        return (int)max;
    }
    
    /** This method draw the mouse coords on the panel
     * @param g The graphics
     **/
    private void drawMouse(Graphics g) {
        g.setColor(java.awt.Color.BLUE);
        g.drawString("(x="+coordToTurn((int)mouseCoord.getX())+",y="+
        coordToPoint((int)mouseCoord.getY())+")",52,12);
        if (cross) {
            g.drawLine(50, (int)mouseCoord.getY(),800,(int)mouseCoord.getY());
            g.drawLine((int)mouseCoord.getX(),0,(int)mouseCoord.getX(),550);
        }
        g.setColor(java.awt.Color.BLACK);
    }
    
    /** This method translate a pixel coord to point (y)
     * @return return a pixel coord to point (y)
     **/
    private int coordToPoint(int y) {
        int ret=0;
        if (y<0) {
            ret=0;
        } else if (y>550) {
            ret=0;
        } else {
            ret=new Double(minY+new Double(550-y).doubleValue()/pxOnePoint.doubleValue()).intValue();
        }
        // 550 doit retourner minY
        // 0 doit retourner maxY
        return ret;
    }
    
    /** This method translate a pixel coord to turn (x)
     * @return return a pixel coord to turn (x)
     **/
    private int coordToTurn(int x) {
        int ret=0;
        if (x<50) {
            ret=0;
        } else if (x>800) {
            ret=0;
        } else {
            ret=new Double(minX+new Double(x-50).doubleValue()/pxOneTurn.doubleValue()).intValue();
        }
        // 0 doit retourner minX
        // 800 doit retourner maxX
        return ret;
    }
    
    /** Draw the zoom rectangle
     * @param g The graphics
     **/
    private void drawRectangle(Graphics g) {
        g.setColor(java.awt.Color.RED);
        int ecartX = (int)(pressedCoord.getX()-mouseCoord.getX());
        int ecartY = (int)(pressedCoord.getY()-mouseCoord.getY());
        if ((ecartX<0)&&(ecartY<0)) {
            // bas droite
            g.drawRect((int)(pressedCoord.getX()), (int)(pressedCoord.getY()),
            -ecartX, -ecartY);
            zoomRectangle = new Rectangle((int)(pressedCoord.getX()),
            (int)(pressedCoord.getY()),
            -ecartX, -ecartY);
        } else if ((ecartX>0)&&(ecartY<0)) {
            // bas gauche
            g.drawRect((int)(pressedCoord.getX()-ecartX),(int)(pressedCoord.getY()),
            ecartX, -ecartY);
            zoomRectangle = new Rectangle((int)(pressedCoord.getX()-ecartX),(int)(pressedCoord.getY()),
            ecartX, -ecartY);
        } else if ((ecartX<0)&&(ecartY>0)) {
            // haut droit
            g.drawRect((int)(pressedCoord.getX()), (int)(pressedCoord.getY()-ecartY),
            -ecartX, ecartY);
            zoomRectangle = new Rectangle((int)(pressedCoord.getX()), (int)(pressedCoord.getY()-ecartY),
            -ecartX, ecartY);
        } else {
            // haut gauche
            g.drawRect((int)(pressedCoord.getX()-ecartX), (int)(pressedCoord.getY()-ecartY),
            ecartX, ecartY);
            zoomRectangle = new Rectangle((int)(pressedCoord.getX()-ecartX), (int)(pressedCoord.getY()-ecartY),
            ecartX, ecartY);
        }
        g.setColor(java.awt.Color.BLACK);
    }
    
    /** Overide of the paint method
     * @param g The graphics
     **/
    public void paint(Graphics g) {
        this.setBackground(new java.awt.Color(255, 255, 255));
        if (repaintCurveNeeded) {
            offscreen = createImage(this.getWidth(), this.getHeight());
            drawCurves(offscreen.getGraphics());
            //drawCurves(g);
            if (offscreen!=null) {// maintenant on affiche notre image
                g.drawImage(offscreen, 0, 0, this );
            }
            repaintCurveNeeded=false;
        }else {
            if (offscreen!=null) {// maintenant on affiche notre image
                g.drawImage(offscreen, 0, 0, this );
            }
        }
        if (grid) {
            drawGrid(g);
        }
        this.setBackground(new java.awt.Color(255, 255, 255));
        drawMouse(g);
        if (pressed) {
            drawRectangle(g);
        }
    }
    
    /** Draw the axes on the panel
     * @param g The graphics
     **/
    private void drawAxes(Graphics g) {
        g.drawLine(50, 550, 50, 0);
        g.drawLine(50, 550, 49, 550);
        g.drawString(""+minY, 20, 550);
        g.drawString(""+maxY,5,10);
        g.drawLine(50,0,49,0);
        // x
        g.drawLine(50, 550, 800, 550);
        g.drawString(""+minX, 50, 570);
        g.drawLine(50, 550, 50, 551);
        g.drawString(""+maxX, 750, 570);
        g.drawLine(800, 550, 800, 551);
        g.drawLine(800, 550, 800, 551);
        g.drawLine(50, 550, 49, 550);
        int midX = 0;
        int midY = 0;
        if (((maxY-minY)%2)==0) {
            midY = new Double(new Double(minY+(maxY-minY)/new Double(2).doubleValue()).doubleValue()).intValue();
        } else {
            midY = new Double(new Double(minY+(maxY-minY+0.5)/new Double(2).doubleValue()).doubleValue()).intValue();
        }
        if (((maxX-minX)%2)==0) {
            midX = new Double(new Double(minX+(maxX-minX)/new Double(2).doubleValue()).doubleValue()).intValue();
        } else {
            midX = new Double(new Double(minX+(maxX-minX+0.5)/new Double(2).doubleValue()).doubleValue()).intValue();
        }
        //lastX=new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue();
        //lastY=new Double(550-pxOnePoint.doubleValue()*(playerData.getData()[i].doubleValue()-minY)).intValue();
        // paire
        //System.out.println("paire");
        g.drawLine(50, (int)(550-(pxOnePoint.doubleValue()*(midY-minY))), 49,
        (int)(550-(pxOnePoint.doubleValue()*(midY-minY))));
        g.drawString(""+midY, 20,
        (int)(550-(pxOnePoint.doubleValue()*(midY-minY))));
        //System.out.println(" : 20,"+(int)(midY*pxOnePoint.doubleValue()));
        //System.out.println("impaire : "+new Double(50+(pxOneTurn.doubleValue()*(midX-minX))).intValue());
        g.drawLine(new Double(50+(pxOneTurn.doubleValue()*(midX-minX))).intValue(), 550,
        new Double(50+(pxOneTurn.doubleValue()*(midX-minX))).intValue(), 551);
        g.drawString(""+midX,
        new Double(50+(pxOneTurn.doubleValue()*(midX-minX))).intValue(),570);
        //System.out.println(" : "+new Double(midX*pxOneTurn.doubleValue()).intValue()+", 570");
    }
    
    /** Draw the grid on the graph
     * @param g The graphics
     **/
    public void drawGrid(Graphics g) {
        // display grid on all 20 px
        int gridX = coordToTurn(100)-minX;
        int gridY = coordToPoint(500)-minY;
        if ((gridX==0)||(gridY==0)) {
            gridX = coordToTurn(150)-minX;
            gridY = coordToPoint(450)-minY;
            if ((gridX==0)||(gridY==0)) {
                return;
            }
            for(int i=(minX);i<=maxX;i+=gridX) {
                g.drawLine(new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue(),0,
                new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue(), 551);
            }
            for(int i=(minY);i<=maxY;i+=gridY) {
                g.drawLine(49, new Double(550-pxOnePoint.doubleValue()*(i-minY)).intValue(),
                800, new Double(550-pxOnePoint.doubleValue()*(i-minY)).intValue());
            }
        } else {
            for(int i=(minX);i<=maxX;i+=gridX) {
                g.drawLine(new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue(),0,
                new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue(), 551);
            }
            for(int i=(minY);i<=maxY;i+=gridY) {
                g.drawLine(49, new Double(550-pxOnePoint.doubleValue()*(i-minY)).intValue(),
                800, new Double(550-pxOnePoint.doubleValue()*(i-minY)).intValue());
            }
        }
    }
    
    /** Define the ratios pxOneTurn an pxOnePoint **/
    private void defineRatio() {
        pxOnePoint = new Double((new Double(550).doubleValue())/(new Double(maxY-minY).doubleValue()));
        pxOneTurn = new Double((new Double(750).doubleValue())/(new Double(maxX-minX).doubleValue()));
    }
    
    /** Fill a white background
     * @param g The graphics
     **/
    private void fillWhite(Graphics g) {
        g.setColor(java.awt.Color.WHITE);
        g.fillRect(0,0,1024,768);
        g.setColor(java.awt.Color.BLACK);
    }
    
    /**
     * Draw the curves of a player on the panel
     * @param g The graphics
     */
    public void drawCurves(Graphics g) {
        //System.out.println("courbe de "+p.toString());
        defineRatio();
        fillWhite(g);
        drawAxes(g);
        if (data.length<=0) {
            return;
        }
        g.setColor(java.awt.Color.GREEN);
        int lastX=50;
        int lastY=new Double(550-pxOnePoint.doubleValue()*(data[minX]-minY)).intValue();
        if (lastY>=550) {
            lastY=550;
        }
        for(int i=(minX+1);i<maxX;i++) {
            if (data[i]<=minY) {
                g.drawLine(lastX, lastY, new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue(),
                550);
                lastX=new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue();
                lastY=550;
            } else {
                g.drawLine(lastX, lastY, new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue(),
                new Double(550-pxOnePoint.doubleValue()*(data[i]-minY)).intValue());
                lastX=new Double(50+pxOneTurn.doubleValue()*(i-minX)).intValue();
                lastY=new Double(550-pxOnePoint.doubleValue()*(data[i]-minY)).intValue();
            }
        }
        g.setColor(java.awt.Color.BLACK);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        setLayout(new java.awt.BorderLayout());

        setBackground(new java.awt.Color(255, 255, 255));
    }//GEN-END:initComponents

    /**
     * Getter for property maxMaxY.
     * @return Value of property maxMaxY.
     */
    public int getMaxMaxY() {
        return maxMaxY;
    }    
    
    /**
     * Setter for property maxMaxY.
     * @param maxMaxY New value of property maxMaxY.
     */
    public void setMaxMaxY(int maxMaxY) {
        this.maxMaxY = maxMaxY;
    }    
    
    /**
     * Getter for property cross.
     * @return Value of property cross.
     */
    public boolean isCross() {
        return cross;
    }
    
    /**
     * Setter for property cross.
     * @param cross New value of property cross.
     */
    public void setCross(boolean cross) {
        this.cross = cross;
    }
    
    /**
     * Getter for property maxY.
     * @return Value of property maxY.
     */
    public int getMaxY() {
        return maxY;
    }
    
    /**
     * Setter for property maxY.
     * @param maxY New value of property maxY.
     */
    public void setMaxY(int maxY) {
        this.maxY = maxY;
    }
    
    /**
     * Getter for property maxMaxX.
     * @return Value of property maxMaxX.
     */
    public int getMaxMaxX() {
        return maxMaxX;
    }
    
    /**
     * Setter for property maxMaxX.
     * @param maxMaxX New value of property maxMaxX.
     */
    public void setMaxMaxX(int maxMaxX) {
        this.maxMaxX = maxMaxX;
    }
    
    /**
     * Getter for property maxX.
     * @return Value of property maxX.
     */
    public int getMaxX() {
        return maxX;
    }
    
    /**
     * Setter for property maxX.
     * @param maxX New value of property maxX.
     */
    public void setMaxX(int maxX) {
        this.maxX = maxX;
    }
    
    /**
     * Getter for property grid.
     * @return Value of property grid.
     */
    public boolean isGrid() {
        return grid;
    }
    
    /**
     * Setter for property grid.
     * @param grid New value of property grid.
     */
    public void setGrid(boolean grid) {
        this.grid = grid;
    }
    
    /**
     * Getter for property minY.
     * @return Value of property minY.
     */
    public int getMinY() {
        return minY;
    }
    
    /**
     * Setter for property minY.
     * @param minY New value of property minY.
     */
    public void setMinY(int minY) {
        this.minY = minY;
    }
    
    /**
     * Getter for property minX.
     * @return Value of property minX.
     */
    public int getMinX() {
        return minX;
    }
    
    /**
     * Setter for property minX.
     * @param minX New value of property minX.
     */
    public void setMinX(int minX) {
        this.minX = minX;
    }
    
    /**
     * Getter for property pressed.
     * @return Value of property pressed.
     */
    public boolean isPressed() {
        return pressed;
    }
    
    /**
     * Setter for property pressed.
     * @param pressed New value of property pressed.
     */
    public void setPressed(boolean pressed) {
        this.pressed = pressed;
    }
    
    /**
     * Getter for property repaintCurveNeeded.
     * @return Value of property repaintCurveNeeded.
     */
    public boolean isRepaintCurveNeeded() {
        return repaintCurveNeeded;
    }
    
    /**
     * Setter for property repaintCurveNeeded.
     * @param repaintCurveNeeded New value of property repaintCurveNeeded.
     */
    public void setRepaintCurveNeeded(boolean repaintCurveNeeded) {
        this.repaintCurveNeeded = repaintCurveNeeded;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
